package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.entity.MonitoringIndicator;
import com.qunzhixinxi.hnqz.admin.service.MonitoringIndicatorService;
import org.springframework.security.access.prepost.PreAuthorize;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * 风险监控
 *
 * @author ryz
 * @date 2021-04-19 16:08:29
 */
@RestController
@AllArgsConstructor
@RequestMapping("/monitoringindicator" )
public class MonitoringIndicatorController {

    private final  MonitoringIndicatorService monitoringIndicatorService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param monitoringIndicator 风险监控
     * @return
     */
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('admin_monitoringindicator_view')" )
    public R getMonitoringIndicatorPage(Page page, MonitoringIndicator monitoringIndicator) {
        return R.ok(monitoringIndicatorService.page(page, Wrappers.query(monitoringIndicator)));
    }


    /**
     * 通过id查询风险监控
     * @param id id
     * @return R
     */
    @GetMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('admin_monitoringindicator_view')" )
    public R getById(@PathVariable("id" ) Long id) {
        return R.ok(monitoringIndicatorService.getById(id));
    }

    /**
     * 新增风险监控
     * @param monitoringIndicator 风险监控
     * @return R
     */
    @SysLog("新增风险监控" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('admin_monitoringindicator_add')" )
    public R save(@RequestBody MonitoringIndicator monitoringIndicator) {
        return R.ok(monitoringIndicatorService.save(monitoringIndicator));
    }

    /**
     * 修改风险监控
     * @param monitoringIndicator 风险监控
     * @return R
     */
    @SysLog("修改风险监控" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('admin_monitoringindicator_edit')" )
    public R updateById(@RequestBody MonitoringIndicator monitoringIndicator) {
        return R.ok(monitoringIndicatorService.updateById(monitoringIndicator));
    }

    /**
     * 通过id删除风险监控
     * @param id id
     * @return R
     */
    @SysLog("通过id删除风险监控" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('admin_monitoringindicator_del')" )
    public R removeById(@PathVariable Long id) {
        return R.ok(monitoringIndicatorService.removeById(id));
    }

}
