package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.entity.SysPortalConf;
import com.qunzhixinxi.hnqz.admin.service.SysPortalConfService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.security.annotation.Inner;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

/**
 * 门户专属配置前端控制器
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2023-06-05 19:28
 */
@RestController
@AllArgsConstructor
public class SysPortalConfController {

	private final SysPortalConfService portalConfService;

	/**
	 * 获取门户专属配置
	 *
	 * @param specialPath 专属路径
	 * @return 专属配置
	 */
	@Inner(value = false)
	@GetMapping(value = "/portal/conf")
	public R<SysPortalConf> getPortalConf(@RequestParam(value = "s") String specialPath) {

		return R.ok(portalConfService.getOne(Wrappers.<SysPortalConf>lambdaQuery().eq(SysPortalConf::getSpecialPath, specialPath)));

	}

	/**
	 * 修改门户专属配置
	 *
	 * @param sysPortalConf 配置实体
	 * @return Boolean
	 */
	@PutMapping(value = "/portal/conf")
	public R<Boolean> editPortalConf(@RequestBody SysPortalConf sysPortalConf) {

		// 主键id校验，防止异常
		if (null == sysPortalConf.getConfId()) {
			R.failed("修改门户配置主键不能为空");
		}


		LocalDateTime now = LocalDateTime.now();
		sysPortalConf.setModifiedBy("admin");
		sysPortalConf.setModifiedTime(now);

		return R.ok(portalConfService.updateById(sysPortalConf));
	}

	/**
	 * 新增门户专属配置
	 *
	 * @param sysPortalConf 配置实体
	 * @return Boolean
	 */
	@PostMapping(value = "/portal/conf")
	public R<Boolean> addPortalConf(@RequestBody SysPortalConf sysPortalConf) {

		LocalDateTime now = LocalDateTime.now();
		sysPortalConf.setCreatedBy("admin");
		sysPortalConf.setModifiedBy("admin");
		sysPortalConf.setCreatedTime(now);
		sysPortalConf.setModifiedTime(now);

		return R.ok(portalConfService.save(sysPortalConf));

	}

	/**
	 * 获取详情
	 *
	 * @param configId 配置详情
	 * @return 详情信息
	 */
	@GetMapping(value = "/portal/conf/details")
	public R<SysPortalConf> getDetails(@RequestParam(value = "configId") Integer configId) {

		SysPortalConf portalConf = portalConfService.getById(configId);

		return R.ok(portalConf);
	}

	/**
	 * 获取分页结果
	 *
	 * @param current 当前页码
	 * @param size    每页记录数
	 * @return 分页结果
	 */
	@GetMapping(value = "/portal/conf/page")
	public R<Page<SysPortalConf>> pageConfigs(@RequestParam(value = "current", defaultValue = "1") Integer current,
											  @RequestParam(value = "size", defaultValue = "20") Integer size) {

		Page<SysPortalConf> confPage = portalConfService.page(new Page<>(current, size));

		return R.ok(confPage);
	}
}