package com.qunzhixinxi.hnqz.admin.controller;

import com.qunzhixinxi.hnqz.admin.entity.UserRecord;
import com.qunzhixinxi.hnqz.admin.service.UserRecordSerivce;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

/**
 * 用户回访记录
 */
@Slf4j
@RestController
@AllArgsConstructor
public class SysUserRecordController {

    private final UserRecordSerivce userRecordSerivce;

    /**
     * 根据UserId查询回访列表
     */
    @SysLog("查询用户回访记录")
    @PreAuthorize("@pms.hasPermission('user_record_check')")
    @GetMapping("/user/record/{userid}")
    public R<?> getUserRecordList(@PathVariable("userid") Integer userId ){
        // 主键id校验，防止异常
        if (null == userId) {
            R.failed("查询回访记录异常，Id为空");
        }

        return R.ok(userRecordSerivce.getUserRecordList(userId));
    }

    /**
     * 根据UserId上传回访记录
     */
    @SysLog("上传用户回访记录")
    @PreAuthorize("@pms.hasPermission('user_record_add')")
    @PostMapping("/user/record")
    public R<?> addUserRecordList(@RequestBody UserRecord userRecord){
        // 主键id校验，防止异常
        if (null != userRecord.getId()) {
            R.failed("新增回访记录异常，不能含有Id");
        }
        if (null != userRecord.getUserId()) {
            R.failed("新增回访记录异常，userid为空");
        }

        return R.ok(userRecordSerivce.addUserRecord(userRecord));
    }

    /**
     * 根据RecordId删除回访记录
     */
    @SysLog("删除用户回访记录")
    @PreAuthorize("@pms.hasPermission('user_record_del')")
    @DeleteMapping("/user/record")
    public R<?> delUserRecordList(@RequestBody UserRecord userRecord){
        // 主键id校验，防止异常
        if (null == userRecord.getId()) {
            R.failed("删除回访记录异常，Id为空");
        }

        return R.ok(userRecordSerivce.delUserRecord(userRecord.getId()));
    }

}
