/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDictItem;
import com.qunzhixinxi.hnqz.admin.entity.WmDaDrugEnt;
import com.qunzhixinxi.hnqz.admin.entity.WmDaPharmacy;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.service.SysDictItemService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.entity.WmDaDrugEntDrugtable;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntDrugtableService;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;


/**
 * 档案-药企药品
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:42
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmdadrugentdrugtable")
public class WmDaDrugEntDrugtableController {

	private final WmDaDrugEntDrugtableService wmDaDrugEntDrugtableService;
	private final WmDaDrugEntService wmDaDrugEntService;
	private final WmScorePackageService wmScorePackageService;
	private final SysDictItemService sysDictItemService;

	/**
	 * 分页查询
	 *
	 * @param page                 分页对象
	 * @param wmDaDrugEntDrugtable 档案-药企药品
	 * @return
	 */
	@GetMapping("/page")
	@PreAuthorize("@pms.hasPermission('admin_wmdadrugentdrugtable_view')")
	public R<IPage<WmDaDrugEntDrugtable>> getWmDaDrugEntDrugtablePage(Page<WmDaDrugEntDrugtable> page, WmDaDrugEntDrugtable wmDaDrugEntDrugtable) {
		OrderItem orderItem = new OrderItem();
		orderItem.setColumn("create_time");
		orderItem.setAsc(false);
		page.addOrder(orderItem);
		QueryWrapper<WmDaDrugEntDrugtable> queryWrapper = Wrappers.query(wmDaDrugEntDrugtable);
		String drugname = wmDaDrugEntDrugtable.getDrugname();
		if (StringUtils.isNotEmpty(wmDaDrugEntDrugtable.getDrugname())) {
			wmDaDrugEntDrugtable.setDrugname(null);
			queryWrapper.like("drugname", drugname);
		} else {
			wmDaDrugEntDrugtable.setDrugname(null);
		}

		String drugnameTy = wmDaDrugEntDrugtable.getDrugnameTy();
		if (StringUtils.isNotEmpty(wmDaDrugEntDrugtable.getDrugnameTy())) {
			wmDaDrugEntDrugtable.setDrugnameTy(null);
			queryWrapper.like("drugname_ty", drugnameTy);
		} else {
			wmDaDrugEntDrugtable.setDrugnameTy(null);
		}
		if (StringUtils.isEmpty(wmDaDrugEntDrugtable.getDrugEntName())) {
			wmDaDrugEntDrugtable.setDrugEntName(null);
		}

		wmDaDrugEntDrugtableService.page(page, queryWrapper);

		if (CollUtil.isNotEmpty(page.getRecords())) {
			// 剂型字典
			List<SysDictItem> doseDictItems = sysDictItemService.list(Wrappers.<SysDictItem>lambdaQuery()
					.eq(SysDictItem::getType, "dose")
					.eq(SysDictItem::getDelFlag, DelEnum.NOT_DEL.val()));
			Map<String, String> doseMap = doseDictItems.stream().collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));
			// 剂型字典
			List<SysDictItem> guigeDictItems = sysDictItemService.list(Wrappers.<SysDictItem>lambdaQuery()
					.eq(SysDictItem::getType, "guige")
					.eq(SysDictItem::getDelFlag, DelEnum.NOT_DEL.val()));
			Map<String, String> guigeMap = guigeDictItems.stream().collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));

			page.getRecords().forEach(record -> {
				if (StrUtil.isNotBlank(record.getDoseId()) && StrUtil.isBlank(record.getDoseName())) {
					record.setDoseName(doseMap.getOrDefault(record.getDoseId(), record.getDoseId()));
				}
				if (StrUtil.isNotBlank(record.getGuigeId()) && StrUtil.isBlank(record.getGuigeName())) {
					record.setGuigeName(guigeMap.getOrDefault(record.getGuigeId(), record.getGuigeId()));
				}
			});
		}

		return R.ok(page);
	}


	/**
	 * 通过id查询档案-药企药品
	 *
	 * @param id id
	 * @return R
	 */
	@GetMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('admin_wmdadrugentdrugtable_view')")
	public R getById(@PathVariable("id") Integer id) {
		WmDaDrugEntDrugtable drugEntDrugtable = wmDaDrugEntDrugtableService.getById(id);

		if (drugEntDrugtable != null) {
			// 剂型字典
			List<SysDictItem> doseDictItems = sysDictItemService.list(Wrappers.<SysDictItem>lambdaQuery()
					.eq(SysDictItem::getType, "dose")
					.eq(SysDictItem::getDelFlag, DelEnum.NOT_DEL.val()));
			Map<String, String> doseMap = doseDictItems.stream().collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));
			// 剂型字典
			List<SysDictItem> guigeDictItems = sysDictItemService.list(Wrappers.<SysDictItem>lambdaQuery()
					.eq(SysDictItem::getType, "guige")
					.eq(SysDictItem::getDelFlag, DelEnum.NOT_DEL.val()));
			Map<String, String> guigeMap = guigeDictItems.stream().collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));

			if (StrUtil.isNotBlank(drugEntDrugtable.getDoseId()) && StrUtil.isBlank(drugEntDrugtable.getDoseName())) {
				drugEntDrugtable.setDoseName(doseMap.getOrDefault(drugEntDrugtable.getDoseId(), drugEntDrugtable.getDoseId()));
			}
			if (StrUtil.isNotBlank(drugEntDrugtable.getGuigeId()) && StrUtil.isBlank(drugEntDrugtable.getGuigeName())) {
				drugEntDrugtable.setGuigeName(guigeMap.getOrDefault(drugEntDrugtable.getGuigeId(), drugEntDrugtable.getGuigeId()));
			}
		}

		return R.ok(drugEntDrugtable);
	}

	/**
	 * 新增档案-药企药品
	 *
	 * @param wmDaDrugEntDrugtable 档案-药企药品
	 * @return R
	 */
	@SysLog("新增档案-药企药品")
	@PostMapping
	@PreAuthorize("@pms.hasPermission('admin_wmdadrugentdrugtable_add')")
	public R save(@RequestBody WmDaDrugEntDrugtable wmDaDrugEntDrugtable) {

		WmDaDrugEntDrugtable query = new WmDaDrugEntDrugtable();
		query.setDrugname(wmDaDrugEntDrugtable.getDrugname());
		List<WmDaDrugEntDrugtable> list = wmDaDrugEntDrugtableService.list(Wrappers.query(query));
		if (list.size() > 0) {
			return R.failed("药品名称已存在");
		}
		WmDaDrugEnt drugEnt = new WmDaDrugEnt();
		List<WmDaDrugEnt> daDrugEnts = wmDaDrugEntService.list(Wrappers.query(drugEnt));
		if (CollectionUtils.isNotEmpty(daDrugEnts)) {
			wmDaDrugEntDrugtable.setDrugEntId(daDrugEnts.get(0).getId());
		}


		wmDaDrugEntDrugtable.setCreateTime(LocalDateTime.now());
		return R.ok(wmDaDrugEntDrugtableService.save(wmDaDrugEntDrugtable));
	}

	/**
	 * 修改档案-药企药品
	 *
	 * @param wmDaDrugEntDrugtable 档案-药企药品
	 * @return R
	 */
	@SysLog("修改档案-药企药品")
	@PutMapping
	@PreAuthorize("@pms.hasPermission('admin_wmdadrugentdrugtable_edit')")
	public R updateById(@RequestBody WmDaDrugEntDrugtable wmDaDrugEntDrugtable) {

		WmDaDrugEntDrugtable oldData = wmDaDrugEntDrugtableService.getById(wmDaDrugEntDrugtable.getId());
		if (!oldData.getDrugEntName().equals(wmDaDrugEntDrugtable.getDrugEntName())) {
			WmDaDrugEnt drugEnt = new WmDaDrugEnt();
			List<WmDaDrugEnt> daDrugEnts = wmDaDrugEntService.list(Wrappers.query(drugEnt));
			if (CollectionUtils.isNotEmpty(daDrugEnts)) {
				wmDaDrugEntDrugtable.setDrugEntId(daDrugEnts.get(0).getId());
			}
		}

//		if(!oldData.getDrugname().equals(wmDaDrugEntDrugtable.getDrugname())){
//			WmDaDrugEntDrugtable query = new WmDaDrugEntDrugtable();
//			query.setDrugname(wmDaDrugEntDrugtable.getDrugname());
//			List<WmDaDrugEntDrugtable> list = wmDaDrugEntDrugtableService.list(Wrappers.query(query));
//			if (list.size() > 0) {
//				return R.failed("药品名称已存在");
//			}
//		}


		return R.ok(wmDaDrugEntDrugtableService.updateById(wmDaDrugEntDrugtable));
	}

	/**
	 * 通过id删除档案-药企药品
	 *
	 * @param id id
	 * @return R
	 */
	@SysLog("通过id删除档案-药企药品")
	@DeleteMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('admin_wmdadrugentdrugtable_del')")
	public R removeById(@PathVariable Integer id) {
		return R.ok(wmDaDrugEntDrugtableService.removeById(id));
	}


	@GetMapping("/list")
	public R listWmDaDrugEntDrugtable() {
		WmDaDrugEntDrugtable wmDaDrugEnt = new WmDaDrugEntDrugtable();
		wmDaDrugEnt.setDelFlag("0");
		wmDaDrugEnt.setEnableFlag("0");

		List<WmDaDrugEntDrugtable> list = wmDaDrugEntDrugtableService.list(Wrappers.query(wmDaDrugEnt));

		if (CollUtil.isNotEmpty(list)) {
			// 剂型字典
			List<SysDictItem> doseDictItems = sysDictItemService.list(Wrappers.<SysDictItem>lambdaQuery()
					.eq(SysDictItem::getType, "dose")
					.eq(SysDictItem::getDelFlag, DelEnum.NOT_DEL.val()));
			Map<String, String> doseMap = doseDictItems.stream().collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));
			// 剂型字典
			List<SysDictItem> guigeDictItems = sysDictItemService.list(Wrappers.<SysDictItem>lambdaQuery()
					.eq(SysDictItem::getType, "guige")
					.eq(SysDictItem::getDelFlag, DelEnum.NOT_DEL.val()));
			Map<String, String> guigeMap = guigeDictItems.stream().collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));

			list.forEach(drugEntDrugtable -> {
				if (StrUtil.isNotBlank(drugEntDrugtable.getDoseId()) && StrUtil.isBlank(drugEntDrugtable.getDoseName())) {
					drugEntDrugtable.setDoseName(doseMap.getOrDefault(drugEntDrugtable.getDoseId(), drugEntDrugtable.getDoseId()));
				}
				if (StrUtil.isNotBlank(drugEntDrugtable.getGuigeId()) && StrUtil.isBlank(drugEntDrugtable.getGuigeName())) {
					drugEntDrugtable.setGuigeName(guigeMap.getOrDefault(drugEntDrugtable.getGuigeId(), drugEntDrugtable.getGuigeId()));
				}
			});
		}

		return R.ok(list);
	}


	@SysLog("新增档案-药企药品")
	@PostMapping("/distribute")
	public R distribute(@RequestBody WmDaDrugEntDrugtable wmDaDrugEntDrugtable) {
		return wmDaDrugEntDrugtableService.distribute(wmDaDrugEntDrugtable);
	}

	@SysLog("获取药品关联部门列表")
	@GetMapping("/listReDept")
	public R listReDept(WmDaDrugEntDrugtable wmDaDrugEntDrugtable) {
		return wmDaDrugEntDrugtableService.listReDept(wmDaDrugEntDrugtable);
	}

	/**
	 * 获取药品生产企业名称列表(已废弃,改为接口/list-drug-table直接获取药品, 兼容历史数据需要用到)
	 *
	 * @param deptId 部门id
	 * @return 名称列表
	 */
	@GetMapping(value = "/list-drug-producer")
	public R listDrugProducerByDeptId(@RequestParam(value = "deptId", required = false) Integer deptId) {
		return R.ok(wmDaDrugEntDrugtableService.listDrugProducerByDeptId(deptId));
	}


	/**
	 * 获取药品列表
	 *
	 * @param drugEntName 部门id
	 * @param packageId 积分包id
	 * @return 名称列表
	 */
	@GetMapping(value = "/list-drug-table")
	public R listDrugByDrugEntName(@RequestParam(value = "drugEntName", required = false) String drugEntName,
								   @RequestParam(value = "packageId") String packageId) {

		WmScorePackage scorePackage = wmScorePackageService.getById(packageId);

		// 兼容历史数据(通过推广药品企业查询药品)
		if (ArrayUtil.isNotEmpty(scorePackage.getDrugProducerList())) {
			return R.ok(wmDaDrugEntDrugtableService.list(Wrappers.<WmDaDrugEntDrugtable>lambdaQuery()
							.in(WmDaDrugEntDrugtable::getDrugEntName, Stream.of(scorePackage.getDrugProducerList()).collect(Collectors.toList())))
					.stream().map(WmDaDrugEntDrugtable::getDrugname).collect(Collectors.toList()));
		} else if (ArrayUtil.isNotEmpty(scorePackage.getDrugtable())) {
			// 查询积分包的推广药品id
			List<WmDaDrugEntDrugtable> drugEntDrugtables =
					wmDaDrugEntDrugtableService.listByIds(Arrays.stream(scorePackage.getDrugtable()).collect(Collectors.toSet()));
			return R.ok(drugEntDrugtables.stream().map(WmDaDrugEntDrugtable::getDrugname).collect(Collectors.toList()));
		}

		return R.ok(Collections.emptyList());
	}

	/**
	 * 获取分配给当前企业的药品列表
	 *
	 * @return {@link R}<{@link ?}> 药品列表
	 */
	@GetMapping("/list-drug")
	public R<?> listDrugByDeptId() {
		Integer deptId = SecurityUtils.getUser().getDeptId();
		return R.ok(wmDaDrugEntDrugtableService.listDrugByDeptId(deptId));
	}

}
