package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.pig4cloud.plugin.excel.annotation.RequestExcel;
import com.qunzhixinxi.hnqz.admin.entity.WmDaPharmacy;
import com.qunzhixinxi.hnqz.admin.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.entity.WmTaskContent;
import com.qunzhixinxi.hnqz.admin.entity.model.excel.BatchPharmacyExcelModel;
import com.qunzhixinxi.hnqz.admin.service.WmDaPharmacyService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskContentService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 档案-药店
 *
 * @author pigx code generator
 * @date 2020-09-23 21:17:34
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/wmdapharmacy")
public class WmDaPharmacyController {

	private final WmDaPharmacyService wmDaPharmacyService;

	private final WmTaskService wmTaskService;

	private final WmTaskContentService wmTaskContentService;

	/**
	 * 分页查询
	 *
	 * @param page         分页对象
	 * @param wmDaPharmacy 档案-药店
	 * @return 分页数据
	 */
	@GetMapping("/page")
	@PreAuthorize("@pms.hasPermission('admin_wmdapharmacy_view')")
	public R<IPage<WmDaPharmacy>> getWmDaPharmacyPage(Page<WmDaPharmacy> page, WmDaPharmacy wmDaPharmacy) {

		LambdaQueryWrapper<WmDaPharmacy> queryWrapper = Wrappers.lambdaQuery();
		if (StrUtil.isNotBlank(wmDaPharmacy.getPharmacyName())) {
			queryWrapper.like(WmDaPharmacy::getPharmacyName, wmDaPharmacy.getPharmacyName());
		}
		queryWrapper.orderByDesc(WmDaPharmacy::getId);

		return R.ok(wmDaPharmacyService.page(page, queryWrapper));
	}


	/**
	 * 分页查询
	 *
	 * @param page         分页对象
	 * @param wmDaPharmacy 档案-医院
	 * @return
	 */
	@GetMapping("/pageAudit")
	public R pageAudit(Page page, WmDaPharmacy wmDaPharmacy) {
		if (null != wmDaPharmacy.getUpdateUser()) {
			WmTask wmTask = wmTaskService.getById(wmDaPharmacy.getUpdateUser());
			if (null != wmTask) {
				WmTaskContent wmTaskContent = wmTaskContentService.getById(wmTask.getTaskContentId());
				if (null != wmTaskContent && null != wmTaskContent.getTemp3()) {
					wmDaPharmacy.setId(wmTaskContent.getTemp3());
				} else {
					wmDaPharmacy.setId("-1");
				}
			} else {
				wmDaPharmacy.setId("-1");
			}
			wmDaPharmacy.setUpdateUser(null);
		}

		return R.ok(wmDaPharmacyService.page(page, Wrappers.query(wmDaPharmacy)));
	}


	/**
	 * 通过id查询档案-药店
	 *
	 * @param id id
	 * @return R
	 */
	@GetMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('admin_wmdapharmacy_view')")
	public R getById(@PathVariable("id") Integer id) {
		return R.ok(wmDaPharmacyService.getById(id));
	}

	/**
	 * 新增档案-药店
	 *
	 * @param wmDaPharmacy 档案-药店
	 * @return R
	 */
	@SysLog("新增档案-药店")
	@PostMapping
	@PreAuthorize("@pms.hasPermission('admin_wmdapharmacy_add')")
	public R save(@RequestBody WmDaPharmacy wmDaPharmacy) {
		int count = wmDaPharmacyService.count(Wrappers.<WmDaPharmacy>lambdaQuery()
				.eq(WmDaPharmacy::getPharmacyName, wmDaPharmacy.getPharmacyName()));

		if (count > 0) {
			return R.failed("药店名称已存在");
		}

		wmDaPharmacy.setCreateTime(LocalDateTime.now());
		wmDaPharmacy.setCreateUser(SecurityUtils.getUser().getId());

		return R.ok(wmDaPharmacyService.save(wmDaPharmacy));
	}

	/**
	 * 修改档案-药店
	 *
	 * @param wmDaPharmacy 档案-药店
	 * @return R
	 */
	@SysLog("修改档案-药店")
	@PutMapping
	@PreAuthorize("@pms.hasPermission('admin_wmdapharmacy_edit')")
	public R updateById(@RequestBody WmDaPharmacy wmDaPharmacy) {

		int count = wmDaPharmacyService.count(Wrappers.<WmDaPharmacy>lambdaQuery()
				.eq(WmDaPharmacy::getPharmacyName, wmDaPharmacy.getPharmacyName())
				.ne(WmDaPharmacy::getId, wmDaPharmacy.getId()));

		if (count > 0) {
			return R.failed("药店名称已存在");
		}

		wmDaPharmacy.setDelFlag("0");
		wmDaPharmacy.setEnableFlag("0");
		wmDaPharmacy.setUpdateUser(SecurityUtils.getUser().getId());
		wmDaPharmacy.setUpdateTime(LocalDateTime.now());

		return R.ok(wmDaPharmacyService.updateById(wmDaPharmacy));
	}

	/**
	 * 通过id删除档案-药店
	 *
	 * @param id id
	 * @return R
	 */
	@SysLog("通过id删除档案-药店")
	@DeleteMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('admin_wmdapharmacy_del')")
	public R removeById(@PathVariable Integer id) {
		return R.ok(wmDaPharmacyService.removeById(id));
	}

	/**
	 * 获取药企列表
	 */
	@GetMapping("/listWmDaPharmacyByName")
	public R listWmDaPharmacyByName(Page page, String name) {
		WmDaPharmacy wmDaHospital = new WmDaPharmacy();
		if (null != name) {
			return R.ok(wmDaPharmacyService.page(page, Wrappers.query(wmDaHospital).like(true, "pharmacy_name", name)));

		} else {
			return R.ok(wmDaPharmacyService.page(page, Wrappers.query(wmDaHospital)));
		}
	}

	/**
	 * 根据名字获取药店列表
	 */
	@GetMapping("/getWmDaPharmacyByName")
	public R getWmDaPharmacyByName(String name) {
		WmDaPharmacy wmDaHospital = new WmDaPharmacy();
		if (StringUtils.isNotEmpty(name)) {
			wmDaHospital.setPharmacyName(name);
			List<WmDaPharmacy> pharmacyList = wmDaPharmacyService.list(Wrappers.query(wmDaHospital));
			if (CollectionUtils.isNotEmpty(pharmacyList)) {
				return R.ok(pharmacyList.get(0));
			}
		}
		return R.ok();
	}

	/**
	 * 批量转换经纬度
	 *
	 * @return
	 */
	@GetMapping("/parseAddress")
	public R<?> parseAddress() {
		wmDaPharmacyService.parseAddress();
		return R.ok();
	}

	/**
	 * 调查信息审核--药企
	 *
	 * @return
	 */
	@GetMapping("/approvalPharmacy")
	public R approvalPharmacy(String taskId) {
		return wmDaPharmacyService.approvalPharmacy(taskId);
	}

	/**
	 * 保存审核意见--药企
	 *
	 * @return R
	 */
	@SysLog("保存审核意见")
	@PostMapping("/updateByApproval")
	public R updateByApproval(@RequestBody Map<String, String> map) {

		return wmDaPharmacyService.updateByapproval(map);
	}

	/**
	 * 药店批量导入
	 *
	 * @param excelModelList 药店列表
	 * @return {@link R}<{@link ?}> 结果
	 */
	@SysLog("药店批量导入")
	@PostMapping(value = "/batch")
	public R<?> batchImport(@RequestExcel List<BatchPharmacyExcelModel> excelModelList) {
		// 参数校验
		if (CollectionUtils.isEmpty(excelModelList)) {
			log.error("空报表，不能上传");
			return R.failed("空报表，不能上传");
		}

		// 实际的处理业务
		Map<String, String> errorMap = wmDaPharmacyService.batchByExcel(excelModelList);

		// 返回上传结果
		Map<String, Object> result = new HashMap<>(3);
		if (CollectionUtil.isEmpty(errorMap)) {
			result.put("code", "SUCCESS");
			result.put("message", "上传成功");
			result.put("data", null);
			return R.ok(result);
		} else {
			log.error("上传数据错误，{}", JSONUtil.toJsonStr(errorMap));
			result.put("code", "FAILURE");
			result.put("message", "上传数据错误");
			result.put("data", errorMap);
			return R.failed(result);
		}
	}

}
