package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.policy.HackLoopTableRenderPolicy;
import com.pig4cloud.plugin.excel.annotation.ResponseExcel;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDictItem;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackageSettleNote;
import com.qunzhixinxi.hnqz.admin.entity.input.WmScorePackageSettleInput;
import com.qunzhixinxi.hnqz.admin.entity.input.WmScorePackageSettleOutput;
import com.qunzhixinxi.hnqz.admin.entity.model.excel.SettlementProofModel;
import com.qunzhixinxi.hnqz.admin.entity.model.excel.WmScorePackageSettleExcelModel;
import com.qunzhixinxi.hnqz.admin.entity.vo.SettleFinancialReviewVO;
import com.qunzhixinxi.hnqz.admin.enums.DeptPermissionEnum;
import com.qunzhixinxi.hnqz.admin.enums.PackageFinishStatusEnum;
import com.qunzhixinxi.hnqz.admin.enums.PackageTypeEnum;
import com.qunzhixinxi.hnqz.admin.enums.SettleStatusEnum;
import com.qunzhixinxi.hnqz.admin.enums.SubjectLocation;
import com.qunzhixinxi.hnqz.admin.mapper.WmScorePackageMapper;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysDictItemService;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.admin.service.WmPayOffService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageSettleNoteService;
import com.qunzhixinxi.hnqz.admin.util.AmountToCNUtil;
import com.qunzhixinxi.hnqz.admin.util.OsEnvUtils;
import com.qunzhixinxi.hnqz.common.core.exception.BizException;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.ding.enums.DingEnum;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 结算表
 *
 * @author Liujh
 * @date 2020-12-1
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/settle")
public class WmPayOffController {

	private final WmPayOffService wmPayOffService;
	private final WmScorePackageService wmScorePackageService;
	private final WmScorePackageMapper wmScorePackageMapper;
	private final WmScorePackageSettleNoteService wmScorePackageSettleNoteService;
	private final SysDeptService sysDeptService;
	private final RedisTemplate<String, Object> redisTemplate;
	private final SysDictItemService dictItemService;
	private final SysUserService sysUserService;

	/**
	 * 财务回退操作
	 *
	 * @param input 操作参数
	 * @return 回退结果
	 */
	@SysLog("提交结算回退")
	@PostMapping("/settleBack")
	public R<?> settleBack(@RequestBody WmScorePackageSettleInput input) {
		if (StringUtils.isEmpty(input.getId())) {
			return R.failed("数据异常");
		}
		return wmPayOffService.settleBack(input);
	}


	/**
	 * 获取批量结算token，三分钟有效
	 * token格式：[租户i]d:order_batch:[用户id]:token:
	 *
	 * @return 结算token
	 */
	@GetMapping(value = "/token")
	public String getOrderBatchToken() {

		String randomStr;

		// 获取操作员
		HnqzUser finaAdmin = SecurityUtils.getUser();
		final String token = String.format("%d:order_batch:%d:token", finaAdmin.getTenantId(), finaAdmin.getId());

		// 返回缓存生成好的
		if (Boolean.TRUE.equals(redisTemplate.hasKey(token))) {
			return (String) redisTemplate.opsForValue().get(token);
		}

		// 生成缓存key,将token缓存到redis
		do {
			randomStr = RandomStringUtils.randomNumeric(6);
		} while (Boolean.FALSE.equals(redisTemplate.opsForValue().setIfAbsent(token, randomStr, 3, TimeUnit.MINUTES)));

		return randomStr;

	}

	/**
	 * 业务提交结算信息到财务
	 *
	 * @param input 结算信息
	 * @return 提交结果
	 */
	@SysLog("提交结算")
	@PostMapping("/settleSubmit")
	public R<?>
	settleSubmit(@RequestBody @Valid WmScorePackageSettleInput input) {
		log.info("提交结算参数:{}", input);

		//1、验证令牌是否合法【令牌的对比和删除必须保证原子性】
		HnqzUser finaAdmin = SecurityUtils.getUser();
		final String REDIS_DEL_SCRIPT = "if redis.call('get', KEYS[1]) == ARGV[1] then return redis.call('del', KEYS[1]) else return 0 end";
		Long execute = redisTemplate.execute(new DefaultRedisScript<>(REDIS_DEL_SCRIPT, Long.class),
				Collections.singletonList(String.format("%d:order_batch:%d:token", finaAdmin.getTenantId(), finaAdmin.getId())), input.getToken());
		// 通过EVAL脚本原子验证令牌和删除令牌
		boolean oops = (execute != null && execute == 0);

		// 令牌验证失败
		if (oops) {
			log.error(String.format("提交 TOKEN 不正确：%s", input.getToken()));
			return R.failed("提交 TOKEN 不正确，请刷新页面重试");
		}

		if (input.getSubjectLocation() == null) {
			return R.failed("人员结算渠道不存在，不能结算");
		}

		return wmPayOffService.settleSubmit(input);
	}

	/**
	 * 业务提交结算信息到财务-MAH(药企的个人承接-其他-结算)
	 *
	 * @param input 结算信息
	 * @return 提交结果
	 */
	@SysLog("MAH提交结算")
	@PostMapping("/settleSubmitForMah")
	public R<?> settleSubmitForMah(@RequestBody @Valid WmScorePackageSettleInput input) {
		log.info("MAH提交结算参数:{}", input);

		//1、验证令牌是否合法【令牌的对比和删除必须保证原子性】
		HnqzUser finaAdmin = SecurityUtils.getUser();
		final String REDIS_DEL_SCRIPT = "if redis.call('get', KEYS[1]) == ARGV[1] then return redis.call('del', KEYS[1]) else return 0 end";
		Long execute = redisTemplate.execute(new DefaultRedisScript<>(REDIS_DEL_SCRIPT, Long.class),
				Collections.singletonList(String.format("%d:order_batch:%d:token", finaAdmin.getTenantId(), finaAdmin.getId())), input.getToken());
		// 通过EVAL脚本原子验证令牌和删除令牌
		boolean oops = (execute != null && execute == 0);

		// 令牌验证失败
		if (oops) {
			log.error(String.format("提交 TOKEN 不正确：%s", input.getToken()));
			return R.failed("提交 TOKEN 不正确，请刷新页面重试");
		}

		if (input.getSubjectLocation() == null) {
			return R.failed("人员结算渠道不存在，不能结算");
		}

		return wmPayOffService.settleSubmitForMah(input);
	}

	/**
	 * 业务保存结算记录
	 *
	 * @param input 结算信息
	 * @return 提交结果
	 */
	@SysLog("保存结算")
	@PostMapping("/settleSave")
	public R<?> settleSave(@RequestBody WmScorePackageSettleInput input) {
		log.info("保存结算参数:{}", input);

		if (CollectionUtils.isEmpty(input.getNotes()) || StringUtils.isEmpty(input.getId()) || input.getSubjectLocation() == null) {

			return R.failed("数据异常或人员结算渠道不存在，不能结算");
		}

		return wmPayOffService.settleSave(input);
	}

	/**
	 * 业务保存结算记录-MAH(药企的个人承接-其他-结算)
	 *
	 * @param input 结算信息
	 * @return 提交结果
	 */
	@SysLog("MAH保存结算")
	@PostMapping("/settleSaveForMah")
	public R<?> settleSaveForMah(@RequestBody WmScorePackageSettleInput input) {
		log.info("MAH保存结算参数:{}", input);

		if (CollectionUtils.isEmpty(input.getNotes()) || StringUtils.isEmpty(input.getId()) || input.getSubjectLocation() == null) {

			return R.failed("数据异常或人员结算渠道不存在，不能结算");
		}

		return wmPayOffService.settleSaveForMah(input);
	}

	/**
	 * 重新结算
	 *
	 * @param input 结算信息
	 * @return 提交结果
	 */
	@SysLog("重新结算")
	@PostMapping("/settleReIssue")
	public R<?> settleReIssue(@RequestBody WmScorePackageSettleInput input) {
		if (StringUtils.isEmpty(input.getId())) {
			return R.failed("数据异常");
		}

		return wmPayOffService.settleReIssueBySubType(input);
	}

	/**
	 * 业务展示结算信息
	 *
	 * @param input 积分包id
	 * @return 待提交的信息
	 */
	@SysLog("结算")
	@PostMapping("/settleById")
	public R<?> settleById(@RequestBody WmScorePackageSettleInput input) {

		if (input == null || StrUtil.isBlank(input.getId())) {
			return R.failed("积分包id缺失");
		}

		return wmPayOffService.settleById(input.getId());
	}

	/**
	 * 多人承接-按积分值业务结算-查询
	 *
	 * @param taskIds 任务id
	 * @return 待提交的信息
	 */
	@SysLog("多人承接-按积分值业务结算-查询")
	@GetMapping("/settle-info-by-taskIds")
	public R<?> settleInfoByTaskIds(@RequestParam("taskIds") String[] taskIds) {

		return wmPayOffService.settleInfoByTaskIds(taskIds);
	}

	/**
	 * 财务审核页面
	 *
	 * @param page  分页信息
	 * @param input 参数
	 * @return 分页结果
	 */
	@SysLog("分页查询结算")
	@GetMapping("/listPackageSettleList")
	public R<?> listPackageSettleList(Page<WmScorePackageSettleOutput> page, WmScorePackageSettleInput input) {

		input.setSendPackageDeptId(String.valueOf(SecurityUtils.getUser().getDeptId()));
		input.setPackageFinishStatus("1");
		input.setTypeid("4,5");
		input.setSettleFlag("1");
		return R.ok(wmScorePackageService.listPackageSettleList(page, input));
	}

	/**
	 * 财务审核页面-用于MAH结算
	 *
	 * @param page  分页信息
	 * @param input 参数
	 * @return 分页结果
	 */
	@SysLog("分页查询结算")
	@GetMapping("/listMahPackageSettleList")
	public R<?> listMAHPackageSettleList(Page<WmScorePackageSettleOutput> page, WmScorePackageSettleInput input) {

		input.setMahSettleDeptId(SecurityUtils.getUser().getDeptId());
		input.setMahSettleStep(UpmsType.MahSettleStep.FINANCE);
		input.setPackageFinishStatus("1");
		input.setTypeid("4");
		input.setSettleFlag("1");
		Page<WmScorePackageSettleOutput> outputPage = wmScorePackageService.listPackageSettleList(page, input);
		outputPage.getRecords().forEach(output -> output.setUserPhones(null));
		return R.ok(outputPage);
	}

	/**
	 * 查询财务结算复核汇总数据
	 *
	 * @param realName 姓名
	 * @param subjectLocation 结算渠道
	 * @return {@link R}<{@link ?}> 结果
	 */
	@GetMapping("/list-financial-review")
	public R<List<SettleFinancialReviewVO>> listFinancialReview(String realName, SubjectLocation subjectLocation, Boolean reviewFlag) {
		Integer deptId = SecurityUtils.getUser().getDeptId();

		SettleFinancialReviewVO query = new SettleFinancialReviewVO();
		query.setRealname(realName);
		query.setSubjectLocation(subjectLocation);
		query.setDeptId(deptId);
		query.setReviewFlag(reviewFlag);
		List<SettleFinancialReviewVO> voList = wmScorePackageMapper.listFinancialReview(query);

		return R.ok(voList);
	}

	/**
	 * 查询财务结算复核汇总数据
	 *
	 * @param page 分页参数
	 * @param userId 用户id
	 * @param subjectLocation 结算渠道
	 * @return {@link R}<{@link ?}> 结果
	 */
	@GetMapping("/page-financial-review-detail")
	public R<IPage<WmScorePackageSettleOutput>> pageFinancialReviewDetail(Page<WmScorePackageSettleOutput> page,
																	@Param("userId") Integer userId,
																	SubjectLocation subjectLocation,
																	Boolean reviewFlag) {
		Integer deptId = SecurityUtils.getUser().getDeptId();

		SettleFinancialReviewVO query = new SettleFinancialReviewVO();
		query.setSubjectLocation(subjectLocation);
		query.setReviewFlag(reviewFlag);
		query.setDeptId(deptId);
		query.setUserId(userId);
		IPage<WmScorePackageSettleOutput> detailPage = wmScorePackageService.pageFinancialReviewDetail(page, query);

		return R.ok(detailPage);
	}

	public R<List<WmScorePackageSettleOutput>> listFinancialReviewDetail(String[] settleNoteIds) {
		List<WmScorePackageSettleNote> settleNotes = wmScorePackageSettleNoteService.listByIds(Arrays.asList(settleNoteIds));

		List<SysUser> sysUsers = sysUserService.listByIds(settleNotes.stream().map(WmScorePackageSettleNote::getUserId).collect(Collectors.toSet()));

		List<WmScorePackageSettleOutput> outputList = new ArrayList<>();
		settleNotes.forEach(settleNote -> {
			WmScorePackageSettleOutput output = new WmScorePackageSettleOutput();
			output.setSettleAmount(settleNote.getSettleAmount());
			output.setActualAmount(settleNote.getActualAmount());
//			output.setUserNames(settleNote.getUserNames());
		});

		// TODO

		return null;
	}

	/**
	 * 结算包结算导出
	 *
	 * @param input 输入参数
	 * @return excel模型
	 */
	@ResponseExcel(name = "exportPackageSettle", sheet = "积分包结算数据")
	@SysLog("积分包结算导出")
	@GetMapping("/exportPackageSettle")
	public List<WmScorePackageSettleExcelModel> exportPackageSettle(WmScorePackageSettleInput input) {
		input.setSendPackageDeptId(String.valueOf(SecurityUtils.getUser().getDeptId()));
		input.setPackageFinishStatus("1");
		input.setTypeid("4");
		input.setSettleFlag("1");
		List<WmScorePackageSettleOutput> list = wmScorePackageService.listAllPackageSettle(input);
		List<WmScorePackageSettleExcelModel> exportList = new ArrayList<>();
		if (CollectionUtil.isEmpty(list)) {
			return Collections.emptyList();
		}

		// 获取所有税源地的发票类目信息
		List<SysDictItem> invoiceCategoryList =
				dictItemService.list(Wrappers.<SysDictItem>lambdaQuery()
						.in(SysDictItem::getType, SubjectLocation.getAllInvoices()));
		Map<String, Map<String, String>> collect = invoiceCategoryList.stream().collect(Collectors.groupingBy(SysDictItem::getType, Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel)));

		// 转换为导出实体
		for (int i = 0; i < list.size(); i++) {
			WmScorePackageSettleOutput settleOutput = list.get(i);
			WmScorePackageSettleExcelModel export = new WmScorePackageSettleExcelModel();
			export.setNumber(i + 1);
			export.setPackageSn(settleOutput.getPackageSn());
			export.setScorePackageName(settleOutput.getScorePackageName());
			export.setPackageType1(PackageTypeEnum.type1Resolve(settleOutput.getPackageType1()).getName2());
			export.setPackageEntName(settleOutput.getPackageEntName());
			export.setCreateTime(settleOutput.getCreateTime());
			export.setPackageFinishStatus(PackageFinishStatusEnum.resolve(settleOutput.getPackageFinishStatus()).getName());
			export.setScore(settleOutput.getScore());

			SubjectLocation subjectLocation = settleOutput.getSubjectLocation();
			if (subjectLocation != null) {
				export.setSubType(subjectLocation.getDescription());
				// 获取当前税源地的发票类目
				Map<String, String> subInvoice = collect.get(subjectLocation.getInvoiceType());
				if (subInvoice != null) {
					// 根据发票类目value获取发票类目名称
					String cate = String.valueOf(settleOutput.getInvoiceCategory());
					String invoice = subInvoice.getOrDefault(cate, "");
					export.setInvoiceCategory(invoice);
				}
			}
			export.setUserNames(settleOutput.getUserNames());
			export.setSettleAmount(settleOutput.getSettleAmount());
			export.setActualAmount(settleOutput.getActualAmount());
			export.setSettleStatus(SettleStatusEnum.resolve(settleOutput.getSettleStatus()).getName());
			export.setBelongDate(settleOutput.getBelongDate());
			exportList.add(export);
		}

		return exportList;
	}

	/**
	 * 结算包结算导出
	 *
	 * @param input 输入参数
	 * @return excel模型
	 */
	@ResponseExcel(name = "exportPackageSettle", sheet = "积分包结算数据")
	@SysLog("积分包结算导出")
	@GetMapping("/exportPackageSettleByMAH")
	public List<WmScorePackageSettleExcelModel> exportPackageSettleByMAH(WmScorePackageSettleInput input) {

		input.setMahSettleDeptId(SecurityUtils.getUser().getDeptId());
		input.setPackageFinishStatus("1");
		input.setTypeid("4");
		input.setSettleFlag("1");
		List<WmScorePackageSettleOutput> list = wmScorePackageService.listPackageSettleList(new Page<>(1, 499), input).getRecords();
		List<WmScorePackageSettleExcelModel> exportList = new ArrayList<>();
		if (CollectionUtil.isEmpty(list)) {
			return Collections.emptyList();
		}

		// 获取所有税源地的发票类目信息
		List<SysDictItem> invoiceCategoryList =
				dictItemService.list(Wrappers.<SysDictItem>lambdaQuery()
						.in(SysDictItem::getType, SubjectLocation.getAllInvoices()));
		Map<String, Map<String, String>> collect = invoiceCategoryList.stream().collect(Collectors.groupingBy(SysDictItem::getType, Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel)));

		// 转换为导出实体
		for (int i = 0; i < list.size(); i++) {
			WmScorePackageSettleOutput settleOutput = list.get(i);
			WmScorePackageSettleExcelModel export = new WmScorePackageSettleExcelModel();
			export.setNumber(i + 1);
			export.setPackageSn(settleOutput.getPackageSn());
			export.setScorePackageName(settleOutput.getScorePackageName());
			export.setPackageType1(PackageTypeEnum.type1Resolve(settleOutput.getPackageType1()).getName2());
			export.setPackageEntName(settleOutput.getPackageEntName());
			export.setCreateTime(settleOutput.getCreateTime());
			export.setPackageFinishStatus(PackageFinishStatusEnum.resolve(settleOutput.getPackageFinishStatus()).getName());
			export.setScore(settleOutput.getScore());

			SubjectLocation subjectLocation = settleOutput.getSubjectLocation();
			if (subjectLocation != null) {
				export.setSubType(subjectLocation.getDescription());
				// 获取当前税源地的发票类目
				Map<String, String> subInvoice = collect.get(subjectLocation.getInvoiceType());
				if (subInvoice != null) {
					// 根据发票类目value获取发票类目名称
					String cate = String.valueOf(settleOutput.getInvoiceCategory());
					String invoice = subInvoice.getOrDefault(cate, "");
					export.setInvoiceCategory(invoice);
				}
			}
			export.setUserNames(settleOutput.getUserNames());
			export.setSettleAmount(settleOutput.getSettleAmount());
			export.setActualAmount(settleOutput.getActualAmount());
			export.setSettleStatus(SettleStatusEnum.resolve(settleOutput.getSettleStatus()).getName());
			export.setBelongDate(settleOutput.getBelongDate());
			exportList.add(export);
		}

		return exportList;
	}

	/**
	 * 导出结算单
	 *
	 * @param packageIds 积分包ID
	 */
	@SysLog("导出结算单")
	@GetMapping("/exportSettlementProof")
	public void exportSettlementProof(@RequestParam("packageIds") List<String> packageIds, HttpServletResponse response) {
		log.info("导出结算单参数packageIds={}", packageIds);

		Integer deptId = SecurityUtils.getUser().getDeptId();
		SysDept dept = sysDeptService.getById(deptId);
		if (!ArrayUtil.contains(dept.getDeptPermissions(), DeptPermissionEnum.SETTLEMENT_PROOF.val())) {
			throw new AccessDeniedException("没有权限");
		}

		// 查询积分包
		List<WmScorePackage> scorePackageList = wmScorePackageService.listByIds(packageIds);
		// 查询结算信息
		List<WmScorePackageSettleNote> settleNoteList =
				wmScorePackageSettleNoteService.list(Wrappers.<WmScorePackageSettleNote>lambdaQuery()
						.in(WmScorePackageSettleNote::getPackageId, packageIds));
		if (CollUtil.isEmpty(settleNoteList)) {
			throw new BizException("结算信息为空");
		}

		Map<String, List<WmScorePackageSettleNote>> settleNoteMap = settleNoteList.stream()
				.collect(Collectors.groupingBy(WmScorePackageSettleNote::getPackageId));

		// 查询企业对应渠道的收款单位信息
//		SysDeptSub queryDeptSub = sysDeptSubService.getOne(Wrappers.<SysDeptSub>lambdaQuery()
//				.eq(SysDeptSub::getDeptId, deptId)
//				.eq(SysDeptSub::getSubjectLocation, subjectLocation)
//				.eq(SysDeptSub::getEnableFlag, SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode()));
//		if (queryDeptSub == null) {
//			throw new RuntimeException("结算渠道配置为空");
//		}

		// 查询结算人员信息
		Set<String> userIds = settleNoteList.stream().map(WmScorePackageSettleNote::getUserId).collect(Collectors.toSet());
		List<SysUser> userList = sysUserService.listByIds(userIds);
		Map<Integer, SysUser> userMap = userList.stream().collect(Collectors.toMap(SysUser::getUserId, Function.identity()));

		// 导出数据整合
		// 积分包列表
		List<SettlementProofModel.PackageDetail> packageDetailList = new ArrayList<>();
		// 申请金额小计
		BigDecimal totalSettleAmount = BigDecimal.ZERO;
		for (int i = 0; i < scorePackageList.size(); i++) {
			WmScorePackage scorePackage = scorePackageList.get(i);
			SettlementProofModel.PackageDetail packageDetail = new SettlementProofModel.PackageDetail();
			packageDetail.setNumber(String.valueOf(i + 1));
			packageDetail.setScorePackageName(scorePackage.getScorePackageName());
			// 结算信息
			List<WmScorePackageSettleNote> settleNotes = settleNoteMap.get(scorePackage.getId());
			if (CollUtil.isEmpty(settleNotes)) {
				continue;
			}
			// 实际到账金额
			BigDecimal settleAmount = settleNotes.stream()
					.map(WmScorePackageSettleNote::getSettleAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
			// 实际结算金额
			BigDecimal actualAmount = settleNotes.stream()
					.map(WmScorePackageSettleNote::getActualAmount)
					.filter(Objects::nonNull)
					.reduce(BigDecimal.ZERO, BigDecimal::add);
			totalSettleAmount = totalSettleAmount.add(actualAmount);
			packageDetail.setSubjectLocation(settleNotes.get(0).getSubjectLocation().getDescription());
			packageDetail.setSettleAmount(String.valueOf(settleAmount));
			if (actualAmount.compareTo(BigDecimal.ZERO) > 0) {
				packageDetail.setActualAmount(String.valueOf(actualAmount));
			}

			SysUser user = userMap.get(Integer.parseInt(settleNotes.get(0).getUserId()));
			// 结算人员
			packageDetail.setTaskUsername(user.getRealname());
			// 开户行
			packageDetail.setBankName(user.getBankName());
			// 银行卡号
			packageDetail.setBankCardNumber(user.getBankCardNumber());

			packageDetailList.add(packageDetail);
		}
		SettlementProofModel settlementProofModel = new SettlementProofModel();
		settlementProofModel.setPackageList(packageDetailList);

//		// 收款单位户名
//		String collectionName = "";
//		// 收款单位账号
//		String collectionAccount = "";
//		// 收款单位开户行
//		String collectionBankName = "";
//		if (queryDeptSub.getSettlementProofInfo() != null) {
//			collectionName = queryDeptSub.getSettlementProofInfo().getOrDefault("collection_name", "");
//			collectionAccount = queryDeptSub.getSettlementProofInfo().getOrDefault("collection_account", "");
//			collectionBankName = queryDeptSub.getSettlementProofInfo().getOrDefault("collection_bank_name", "");
//		}

		// 服务器模板文件路径
		String resourcePath = OsEnvUtils.getEachEnvPaths().get("EXCEL_FILE_PATH") + "settlement_application_form_template.docx";

		HackLoopTableRenderPolicy policy = new HackLoopTableRenderPolicy();

		Configure config = Configure.newBuilder().bind("package_list", policy).build();

//		String finalCollectionName = collectionName;
//		String finalCollectionAccount = collectionAccount;
//		String finalCollectionBankName = collectionBankName;
		BigDecimal finalTotalSettleAmount = totalSettleAmount;
		XWPFTemplate template = XWPFTemplate.compile(resourcePath, config).render(
				new HashMap<String, Object>() {{
					// 积分包列表
					put("package_list", settlementProofModel.getPackageList());
					if (finalTotalSettleAmount.compareTo(BigDecimal.ZERO) > 0) {
						// 申请金额小计
						put("total_settle_amount", String.valueOf(finalTotalSettleAmount));
						// 金额大写
						put("total_settle_amount_cn", AmountToCNUtil.number2CNMontrayUnit(finalTotalSettleAmount));
					}
//					put("collection_name", finalCollectionName);
//					put("collection_account", finalCollectionAccount);
//					put("collection_bank_name", finalCollectionBankName);
				}}
		);

		try {
			//清空response
			response.reset();
			//设置response的Header
			response.addHeader("Content-Disposition",
					String.format("attachment;filename=请款申请单(%s)-%s.docx", String.join("_", packageIds), DateUtil.format(LocalDateTime.now(),
							DatePattern.PURE_DATETIME_PATTERN)));
			response.setContentType("application/vnd.openxmlformats-officedocument.wordprocessingml.document");
			OutputStream os = new BufferedOutputStream(response.getOutputStream());

			template.write(os);

			os.flush();
			os.close();
		} catch (IOException e) {
			log.error("导出失败", e);
			throw new RuntimeException("导出失败");
		}
	}

	@SysLog("结算")
	@PostMapping("/saveSettleInfo")
	public R<?> saveSettleInfo(@RequestBody List<WmScorePackage> wsList) {

		if (CollectionUtils.isEmpty(wsList)) {
			return R.failed("请选择数据");
		}

		Optional<WmScorePackage> any = wsList.stream().filter(ws -> StringUtils.isEmpty(ws.getId())).findAny();

		if (any.isPresent()) {
			return R.failed("数据异常，请联系管理员：" + any.get().getScoreName());
		}

		return wmPayOffService.saveSettleInfo(wsList);
	}

	/**
	 * 结算
	 *
	 * @param input 结算信息
	 * @return 结算操作结果
	 */
	@SysLog("结算-自由选择渠道")
	@PostMapping("/settleIssueBySubType")
	public R<?> settleIssueBySubType(@RequestBody WmScorePackageSettleInput input) {


		if (StrUtil.isBlank(input.getId()) || StrUtil.isBlank(input.getScorePackageName()) || input.getSubjectLocation() == null || CollUtil.isEmpty(input.getNotes()) || null == input.getInvoiceCategory()) {
			return R.failed("数据异常");
		}

		return wmPayOffService.settleIssueBySubType(input);
	}

	/**
	 * 批量财务结算
	 *
	 * @param inputList 批量结算
	 * @return 结算操作结果
	 */
	@SysLog("批量财务结算")
	@PostMapping("/batch-financial-settle")
	@SuppressWarnings("all")
	public R<?> batchFinancialSettle(@RequestBody List<WmScorePackageSettleInput> inputList) {

		if (CollectionUtil.isEmpty(inputList)) {
			return R.failed("数据异常");
		}

		List<Map<String, String>> msgList = new ArrayList<>();

		// 获取当日所在月份
//		LocalDateTime today = LocalDateTime.now();
//		int monthValue = today.getMonthValue();

		for (WmScorePackageSettleInput input : inputList) {

			if (StringUtils.isEmpty(input.getId()) || null == input.getSubjectLocation() || null == input.getInvoiceCategory() || StringUtils.isEmpty(input.getScorePackageName())) {
				return R.failed("数据异常");
			}

			// 查询结算详情（用于拼接notes数据）
			R<?> queryR = wmPayOffService.settleById(input.getId());

			if (queryR.getCode() == 0) {
				Map<String, Object> queryData = (Map<String, Object>) queryR.getData();
				List<WmScorePackageSettleNote> notes = (List<WmScorePackageSettleNote>) queryData.get("notes");

//				Optional<WmScorePackageSettleNote> checkMonthOpt = notes.stream().filter(note -> note.getSubTime() == null || note.getSubTime().getMonthValue() != monthValue).findAny();
//				if (checkMonthOpt.isPresent()) {
//					return R.failed("不支持结算非当月提交记录，请回退并重新提交后重试");
//				}

				input.setNotes(notes);
				// 调用单个结算
				R<?> settleR = wmPayOffService.settleIssueBySubType(input);
				boolean failFlag;
				if (settleR.getCode() == 0) {
					// 结算成功，不进行页面提示
					Optional<WmScorePackageSettleNote> any = notes.stream().filter(note -> !DingEnum.NOTE_STATUS_SUBMIT.getType().equals(note.getSettleNoteStatus())).findAny();
					failFlag = any.isPresent();
				} else {
					failFlag = true;
				}

				if (failFlag) {
					Map<String, String> msgMap = new HashMap<>(2);
					msgMap.put("packageName", input.getScorePackageName());
					msgMap.put("reason", settleR.getMsg());
					msgList.add(msgMap);
				}

			} else {
				Map<String, String> msgMap = new HashMap<>(2);
				msgMap.put("packageName", input.getScorePackageName());
				msgMap.put("reason", queryR.getMsg());
				msgList.add(msgMap);
			}
		}

		return R.ok(msgList);
	}

	/**
	 * 手动回调
	 *
	 * @param input 修改的对象信息
	 * @return 结算操作结果
	 */
	@SysLog("手动回调")
	@PostMapping("/notify-by-oneself")
	public R<Boolean> notifyByOneself(@RequestBody WmScorePackageSettleNote input) {
		// 开始输出
		log.info("===== 手工回调开始 =====");

		// 初始化变量命名
		Integer settleNoteId = input.getId();
		LocalDateTime notifyTime = input.getNotifyTime();
		Integer settleNoteStatus = input.getSettleNoteStatus();

		// 初始化内部参数
		Integer userid = SecurityUtils.getUser().getId();
		String remark = ("手动回调，操作人" + userid);

		// 主键id校验，防止异常
		if (null == settleNoteId) {
			R.failed("积分包ID不能为空");
		}

		// 信息校验，防止异常
		if (null == notifyTime ||
				null == settleNoteStatus
		) {
			R.failed("结算修改信息不能为空");
		}

		// 校验数据有效性
		// 查询结算中的数据
		List<WmScorePackageSettleNote> settleNoteByidList = wmScorePackageSettleNoteService.getSettleNoteById(input.getId(), Integer.parseInt(SettleStatusEnum.SETTLE_STATUS_SUBMIT.getVal()));
		log.info("结算信息查询：{}", settleNoteByidList);
		if (CollUtil.isEmpty(settleNoteByidList)) {
			return R.failed("修改的目标数据状态异常，不能修改");
		}

		String packageId = settleNoteByidList.get(0).getPackageId();

		// 更新结算数据
		boolean updateSettleResult = wmScorePackageSettleNoteService.updateSettleNoteById(packageId, settleNoteId, settleNoteStatus, remark, notifyTime);

		// 结束输出
		log.info("===== 手工回调开始 =====");

		// 返回更新结果
		return R.ok(updateSettleResult);
	}

}
