package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.entity.WmDaDrugEntDrugtable;
import com.qunzhixinxi.hnqz.admin.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntDrugtableService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.entity.WmWkAlbum;
import com.qunzhixinxi.hnqz.admin.service.WmWkAlbumService;
import org.springframework.security.access.prepost.PreAuthorize;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * 文库-相册
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:24
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmwkalbum" )
public class WmWkAlbumController {

    private final  WmWkAlbumService wmWkAlbumService;
	private final WmDaDrugEntDrugtableService wmDaDrugEntDrugtableService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmWkAlbum 文库-相册
     * @return
     */
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('admin_wmwkalbum_view')" )
    public R getWmWkAlbumPage(Page page, WmWkAlbum wmWkAlbum) {
		OrderItem orderItem=new OrderItem();
		orderItem.setColumn("create_time");
		orderItem.setAsc(false);
		page.addOrder(orderItem);

		QueryWrapper queryWrapper=Wrappers.query();
		if(null!=wmWkAlbum.getXcmc())
		{
			queryWrapper.like(true,"xcmc",wmWkAlbum.getXcmc());
		}

		if(null!=wmWkAlbum.getGlyp())
		{
			queryWrapper.apply(true,"FIND_IN_SET ("+wmWkAlbum.getGlyp()+",glyp)");
		}

		queryWrapper.eq("del_flag", DelEnum.NOT_DEL.val());
		queryWrapper.orderByDesc("create_time");

    	Page pageAnswer =wmWkAlbumService.page(page, queryWrapper);
    	List<WmWkAlbum> wmWkAlbumList=pageAnswer.getRecords();
    	if(null!=wmWkAlbumList)
		{
			for (WmWkAlbum wmW :wmWkAlbumList) {
				if(null!=wmW.getGlyp())
				{
					WmDaDrugEntDrugtable wmDaDrugEntDrugtable=new WmDaDrugEntDrugtable();
//					if(wmW.getGlyp().contains(","))
//					{
						String [] ar=wmW.getGlyp().split(",");
						List<String> list=new ArrayList<>(Arrays.asList(ar));
						List<WmDaDrugEntDrugtable>  wmDaDrugEntDrugtableList=wmDaDrugEntDrugtableService.listByIds(list);
						if(wmDaDrugEntDrugtableList.size()>0)
						{
							String temp1="";
							for (int i=0;i<wmDaDrugEntDrugtableList.size();i++) {
								temp1+=wmDaDrugEntDrugtableList.get(i).getDrugname();
								if(i<wmDaDrugEntDrugtableList.size()-1)
								{
									temp1+=",";
								}
							}
							wmW.setTemp1(temp1);
						}

//					}
				}
			}
		}
		pageAnswer.setRecords(wmWkAlbumList);
        return R.ok(pageAnswer);
    }


    /**
     * 通过id查询文库-相册
     * @param id id
     * @return R
     */
    @GetMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('admin_wmwkalbum_view')" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmWkAlbumService.getById(id));
    }

    /**
     * 新增文库-相册
     * @param wmWkAlbum 文库-相册
     * @return R
     */
    @SysLog("新增文库-相册" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('admin_wmwkalbum_add')" )
    public R save(@RequestBody WmWkAlbum wmWkAlbum) {
		wmWkAlbum.setForwardNum(0);
		wmWkAlbum.setCreateTime(LocalDateTime.now());
//        return R.ok(wmWkAlbumService.save(wmWkAlbum));
		wmWkAlbumService.saveWmWkAlbum(wmWkAlbum);
        return R.ok();
    }

    /**
     * 修改文库-相册
     * @param wmWkAlbum 文库-相册
     * @return R
     */
    @SysLog("修改文库-相册" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('admin_wmwkalbum_edit')" )
    public R updateById(@RequestBody WmWkAlbum wmWkAlbum) {
		wmWkAlbumService.updateWmWkAlbumById(wmWkAlbum);
        return R.ok();
    }

    /**
     * 通过id删除文库-相册
     * @param id id
     * @return R
     */
    @SysLog("通过id删除文库-相册" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('admin_wmwkalbum_del')" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmWkAlbumService.removeById(id));
    }

}
