package com.qunzhixinxi.hnqz.admin.entity.dto;

import cn.hutool.core.util.StrUtil;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.Getter;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;

/**
 * 企业任务请求参数DTO
 *
 * @author lixuesong
 * @date 2023/09/21
 */
@Data
public class DeptTaskDTO implements Serializable {
    private static final long serialVersionUID = 410601826934557948L;

    /**
     * 任务类型61-企业通用
     *
     * @author lixuesong
     * @date 2023/09/21
     */
    @Data
    public static class TaskType61 implements Serializable {
        private static final long serialVersionUID = -8969888154230924285L;

        /**
         * 积分包id
         */
        @NotBlank(message = "积分包id必填")
        private String packageId;

        /**
         * 主题
         */
        @NotBlank(message = "主题必填")
        private String meetingtTitle;

        /**
         * 时间
         */
        @NotBlank(message = "时间必填")
        private String meetingTime;

        /**
         * 文件上传
         */
        @NotEmpty(message = "文件上传必填")
        private List<FileUrlObj> files;

        /**
         * 外采费用
         */
        private String outsourcingAmount;

        /**
         * 外采费用-凭证
         */
        private List<FileUrlObj> outsourcingAmountProof;

        /**
         * 任务积分
         */
        @NotNull(message = "任务积分必填")
        private Integer score;

        /**
         * 会议总结
         */
        private String meetingSummary;
    }

    /**
     * 任务类型62-医学会各级年会及卫星会
     *
     * @author lixuesong
     * @date 2023/09/21
     */
    @Data
    public static class TaskType62 implements Serializable {
        private static final long serialVersionUID = -2282200603021121878L;

        /**
         * 积分包id
         */
        @NotBlank(message = "积分包id必填")
        private String packageId;

        /**
         * 会议主题
         */
        @NotBlank(message = "会议主题必填")
        private String meetingtTitle;

        /**
         * 会议时间
         */
        @NotBlank(message = "会议时间必填")
        private String meetingTime;

        /**
         * 会议人次
         */
        @NotBlank(message = "会议人次必填")
        private String meetingNumber;

        /**
         * 会议邀请函
         */
        @NotEmpty(message = "会议邀请函必填")
        private List<FileUrlObj> meetingInvitationUrl;

        /**
         * 会议计划表
         */
        @NotEmpty(message = "会议计划表必填")
        private List<FileUrlObj> meetingPlanUrl;

        /**
         * 会议签到表
         */
        @NotEmpty(message = "会议签到表必填")
        private List<FileUrlObj> meetingCheckInUrl;

        /**
         * 现场照片
         */
        @NotEmpty(message = "现场照片必填")
        private List<FileUrlObj> livePhotosUrl;

        /**
         * 会议内容
         */
        @NotEmpty(message = "会议内容必填")
        private List<FileUrlObj> meetingContentUrl;

        /**
         * 外采费用
         */
        private String outsourcingAmount;

        /**
         * 外采费用-凭证
         */
        private List<FileUrlObj> outsourcingAmountProof;

        /**
         * 会议总结
         */
        private String meetingSummary;
    }

    /**
     * 任务类型63-专家共识巡讲会
     *
     * @author lixuesong
     * @date 2023/09/21
     */
    @Data
    public static class TaskType63 implements Serializable {
        private static final long serialVersionUID = -1918885625462400849L;

        /**
         * 积分包id
         */
        @NotBlank(message = "积分包id必填")
        private String packageId;

        /**
         * 主题
         */
        @NotBlank(message = "主题必填")
        private String meetingtTitle;

        /**
         * 时间
         */
        @NotBlank(message = "时间必填")
        private String meetingTime;

        /**
         * 参加人次
         */
        @NotBlank(message = "参加人次必填")
        private String meetingNumber;

        /**
         * 活动策划书
         */
        @NotEmpty(message = "活动策划书必填")
        private List<FileUrlObj> eventPlannerUrl;

        /**
         * 活动计划表
         */
        @NotEmpty(message = "活动计划表必填")
        private List<FileUrlObj> meetingPlanUrl;

        /**
         * 会议签到表
         */
        @NotEmpty(message = "会议签到表必填")
        private List<FileUrlObj> meetingCheckInUrl;

        /**
         * 现场照片
         */
        @NotEmpty(message = "现场照片必填")
        private List<FileUrlObj> livePhotosUrl;

        /**
         * 会议内容
         */
        @NotEmpty(message = "会议内容必填")
        private List<FileUrlObj> meetingContentUrl;

        /**
         * 外采费用
         */
        private String outsourcingAmount;

        /**
         * 外采费用-凭证
         */
        private List<FileUrlObj> outsourcingAmountProof;

        /**
         * 会议总结
         */
        private String meetingSummary;
    }

	/**
	 * 任务类型66-医院准入
	 *
	 * @author lixuesong
	 * @date 2023/11/10
	 */
	@Data
	public static class TaskType66 implements Serializable {
		private static final long serialVersionUID = -7268001509315036318L;

		/**
		 * 积分包id
		 */
		@NotBlank(message = "积分包id必填")
		private String packageId;

		/**
		 * 推广产品名称
		 */
		@NotBlank(message = "推广产品名称必填")
		private String productName;

		/**
		 * 医院等级
		 */
		@NotBlank(message = "医院等级必填")
		private String hospitalLevel;

		/**
		 * 产品进入形式
		 */
		@NotBlank(message = "产品进入形式必填")
		private String entryForm;

		/**
		 * 预期销售量
		 */
		@NotBlank(message = "预期销售量必填")
		private String expectedSales;

		/**
		 * 负责人
		 */
		@NotBlank(message = "负责人必填")
		private String leader;

		/**
		 * 中标通知书
		 */
		@NotEmpty(message = "中标通知书必填")
		private List<FileUrlObj> noticeUrl;

		/**
		 * 购销合同
		 */
		@NotEmpty(message = "购销合同必填")
		private List<FileUrlObj> contractsUrl;

		/**
		 * 合同金额
		 */
		@NotNull(message = "合同金额必填")
		private BigDecimal contractsAmount;

		/**
		 * 发票上传
		 */
		@NotEmpty(message = "发票上传必填")
		private List<FileUrlObj> invoiceUrl;

		/**
		 * 会议总结
		 */
		private String summary;
	}

	/**
	 * 任务类型67-招投标服务
	 *
	 * @author lixuesong
	 * @date 2023/11/10
	 */
	@Data
	public static class TaskType67 implements Serializable {
		private static final long serialVersionUID = -9203744525584265085L;

		/**
		 * 积分包id
		 */
		@NotBlank(message = "积分包id必填")
		private String packageId;

		/**
		 * 推广产品名称
		 */
		@NotBlank(message = "推广产品名称必填")
		private String productName;

		/**
		 * 招标计划书
		 */
		@NotEmpty(message = "招标计划书必填")
		private List<FileUrlObj> tenderProposalUrl;

		/**
		 * 招标目录
		 */
		@NotEmpty(message = "招标目录必填")
		private List<FileUrlObj> tendersCatalogueUrl;

		/**
		 * 标书
		 */
		@NotEmpty(message = "标书必填")
		private List<FileUrlObj> tenderUrl;

		/**
		 * 投标资料清单
		 */
		@NotEmpty(message = "投标资料清单必填")
		private List<FileUrlObj> informationListUrl;

		/**
		 * 产品投标及报价
		 */
		@NotEmpty(message = "产品投标及报价必填")
		private List<FileUrlObj> quoteUrl;

		/**
		 * 招标活动总结表
		 */
		@NotEmpty(message = "招标活动总结表必填")
		private List<FileUrlObj> summaryTableUrl;

		/**
		 * 会议总结
		 */
		private String summary;
	}

	/**
	 * 任务类型69-商业渠道维护
	 *
	 * @author lixuesong
	 * @date 2023/11/10
	 */
	@Data
	public static class TaskType68 implements Serializable {
		private static final long serialVersionUID = 3437372981272089491L;

		/**
		 * 积分包id
		 */
		@NotBlank(message = "积分包id必填")
		private String packageId;

		/**
		 * 推广产品名称
		 */
		@NotBlank(message = "推广产品名称必填")
		private String productName;

		/**
		 * 会议时间
		 */
		@NotBlank(message = "会议时间必填")
		private String meetingTime;

		/**
		 * 会议地点
		 */
		@NotBlank(message = "会议地点必填")
		private String meetingPlace;

		/**
		 * 会议计划表
		 */
		@NotEmpty(message = "会议计划表必填")
		private List<FileUrlObj> meetingPlanUrl;

		/**
		 * 会议通知/邀请函
		 */
		@NotEmpty(message = "会议通知/邀请函必填")
		private List<FileUrlObj> meetingInvitationUrl;

		/**
		 * 签到表
		 */
		@NotEmpty(message = "签到表必填")
		private List<FileUrlObj> meetingCheckInUrl;

		/**
		 * 会议照片
		 */
		@NotEmpty(message = "会议照片必填")
		private List<FileUrlObj> meetingPhotosUrl;

		/**
		 * 宣传资料使用清单
		 */
		@NotEmpty(message = "宣传资料使用清单必填")
		private List<FileUrlObj> promotionalMaterialsUrl;

		/**
		 * 发票上传
		 */
		@NotEmpty(message = "发票上传必填")
		private List<FileUrlObj> invoiceUrl;

		/**
		 * 会议纪要或总结
		 */
		private String summary;
	}

	/**
	 * 任务类型69-商业渠道维护
	 *
	 * @author lixuesong
	 * @date 2023/11/10
	 */
	@Data
	public static class TaskType69 implements Serializable {
		private static final long serialVersionUID = -2615877505967750457L;

		/**
		 * 积分包id
		 */
		@NotBlank(message = "积分包id必填")
		private String packageId;

		/**
		 * 推广产品名称
		 */
		@NotBlank(message = "推广产品名称必填")
		private String productName;

		/**
		 * 产品规格
		 */
		@NotBlank(message = "产品规格必填")
		private String productSpecifications;

		/**
		 * 维护周期
		 */
		@NotBlank(message = "维护周期必填")
		private String maintenanceIntervals;

		/**
		 * 进量
		 */
		@NotBlank(message = "进量必填")
		private String purchaseNumber;

		/**
		 * 销量
		 */
		@NotBlank(message = "销量必填")
		private String salesNumber;

		/**
		 * 存量
		 */
		@NotBlank(message = "存量必填")
		private String stockNumber;

		/**
		 * 本产品信息搜集表
		 */
		@NotEmpty(message = "本产品信息搜集表必填")
		private List<FileUrlObj> productInformationUrl;

		/**
		 * 商业流向数据表
		 */
		@NotEmpty(message = "商业流向数据表必填")
		private List<FileUrlObj> businessFlowDataUrl;

		/**
		 * 竞品信息搜集表
		 */
		@NotEmpty(message = "竞品信息搜集表必填")
		private List<FileUrlObj> competitorInformationUrl;

		/**
		 * 竞品商业数据流向表
		 */
		@NotEmpty(message = "竞品商业数据流向表必填")
		private List<FileUrlObj> competitorBusinessDataFlowUrl;

		/**
		 * 总结
		 */
		private String summary;
	}

	/**
	 * 任务类型70-第三方专项合同
	 *
	 * @author lixuesong
	 * @date 2023/11/10
	 */
	@Data
	public static class TaskType70 implements Serializable {
		private static final long serialVersionUID = -4614288064808194504L;

		/**
		 * 积分包id
		 */
		@NotBlank(message = "积分包id必填")
		private String packageId;

		/**
		 * 项目名称
		 */
		@NotBlank(message = "项目名称必填")
		private String projectName;

		/**
		 * 甲方名称
		 */
		@NotBlank(message = "甲方名称必填")
		private String partAName;

		/**
		 * 乙方名称
		 */
		@NotBlank(message = "乙方名称必填")
		private String partBName;

		/**
		 * 项目简介
		 */
		@NotBlank(message = "项目简介必填")
		private String introduction;

		/**
		 * 合同金额
		 */
		@NotNull(message = "合同金额必填")
		private BigDecimal contractsAmount;

		/**
		 * 合同上传
		 */
		@NotEmpty(message = "合同上传必填")
		private List<FileUrlObj> contractsUrl;

		/**
		 * 发票上传
		 */
		@NotEmpty(message = "发票上传必填")
		private List<FileUrlObj> invoiceUrl;

		/**
		 * 总结
		 */
		private String summary;
	}

    /**
     * 文件 URL obj
     *
     * @author lixuesong
     * @date 2023/09/27
     */
    @Data
    public static class FileUrlObj implements Serializable {
        private static final long serialVersionUID = 7914265352319844404L;

        private String fileName;
        
        private String url;
    }

	/**
	 * 任务类型参数枚举
	 *
	 * @author lixuesong
	 * @date 2023/11/10
	 */
	@Getter
	@AllArgsConstructor
	public enum DeptTaskTypeParam {
		TASK_TYPE_61("61", "企业通用", TaskType61.class),
		TASK_TYPE_62("62", "医学会各级年会及卫星会", TaskType62.class),
		TASK_TYPE_63("63", "专家共识巡讲会", TaskType63.class),
		TASK_TYPE_66("66", "医院准入", TaskType66.class),
		TASK_TYPE_67("67", "招投标服务", TaskType67.class),
		TASK_TYPE_68("68", "商业推广会", TaskType68.class),
		TASK_TYPE_69("69", "商业渠道维护", TaskType69.class),
		TASK_TYPE_70("70", "第三方专项合同", TaskType70.class);

		/**
		 * 类型
		 */
		private final String type;

		/**
		 * 描述
		 */
		private final String desc;

		/**
		 * class类型
		 */
		private final Class<?> clazz;

		/**
		 * 根据任务类型id查询枚举对象
		 *
		 * @param type 任务类型id
		 * @return {@link DeptTaskTypeParam} 枚举对象
		 */
		public static DeptTaskTypeParam getByType(String type) {
			for (DeptTaskTypeParam taskTypeParam : DeptTaskTypeParam.values()) {
				if (StrUtil.equals(taskTypeParam.getType(), type)) {
					return taskTypeParam;
				}
			}
			return null;
		}
	}

}
