package com.qunzhixinxi.hnqz.admin.oe.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.oe.entity.vo.OeTenantVO;
import com.qunzhixinxi.hnqz.admin.oe.entity.dto.OeTenantRequest;
import com.qunzhixinxi.hnqz.admin.oe.entity.vo.OeTerminalVO;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 租户管理
 */
@Slf4j
@RestController
@RequestMapping("/oe/tenant")
public class OeTenantController {

    /**
     * 保存药企信息
     *
     * @param toSaveDrugCompany 参数
     * @return {@link R}<{@link ?}>
     */
    @SysLog("OE保存药企信息")
    @PostMapping("/drug-company/save")
    public R<?> saveDrugCompany(@RequestBody OeTenantRequest.ToSaveDrugCompany toSaveDrugCompany) {
        log.info("OE保存药企信息参数:{}", toSaveDrugCompany);
        return R.ok();
    }

    /**
     * 更新药企信息
     *
     * @param toUpdateDrugCompany 参数
     * @return {@link R}<{@link ?}>
     */
    @SysLog("OE更新药企信息")
    @PostMapping("/drug-company/update")
    public R<?> updateDrugCompany(@RequestBody OeTenantRequest.ToUpdateDrugCompany toUpdateDrugCompany) {
        log.info("OE更新药企信息参数:{}", toUpdateDrugCompany);
        return R.ok();
    }

    /**
     * 启用/停用药企/代理商/终端
     *
     * @param toLockUnlock 参数
     * @return {@link R}<{@link ?}>
     */
    @SysLog("OE启用/停用药企/代理商/终端")
    @PostMapping("/lock-unlock")
    public R<?> updateLockUnlock(@RequestBody OeTenantRequest.ToLockUnlock toLockUnlock) {
        log.info("OE启用/停用tenant参数:{}", toLockUnlock);
        return R.ok();
    }

    /**
     * 保存代理商信息
     *
     * @param toSaveCso 参数
     * @return {@link R}<{@link ?}>
     */
    @SysLog("OE保存代理商信息")
    @PostMapping("/cso/save")
    public R<?> saveCso(@RequestBody OeTenantRequest.ToSaveCso toSaveCso) {
        log.info("OE保存代理商信息参数:{}", toSaveCso);
        return R.ok();
    }

    /**
     * 更新代理商信息
     *
     * @param toUpdateCso 参数
     * @return {@link R}<{@link ?}>
     */
    @SysLog("OE更新代理商信息")
    @PostMapping("/cso/update")
    public R<?> saveCso(@RequestBody OeTenantRequest.ToUpdateCso toUpdateCso) {
        log.info("OE更新代理商信息参数:{}", toUpdateCso);
        return R.ok();
    }


    /**
     * 保存终端
     *
     * @param toSaveTerminal 参数
     * @return {@link R}<{@link ?}>
     */
    @SysLog("OE保存终端信息")
    @PostMapping("/terminal/save")
    public R<?> saveTerminal(@RequestBody OeTenantRequest.ToSaveTerminal toSaveTerminal) {
        log.info("OE保存终端信息参数:{}", toSaveTerminal);
        return R.ok();
    }

    /**
     * 更新终端
     *
     * @param toUpdateTerminal 参数
     * @return {@link R}<{@link ?}>
     */
    @SysLog("OE更新终端信息")
    @PostMapping("/terminal/update")
    public R<?> saveTerminal(@RequestBody OeTenantRequest.ToUpdateTerminal toUpdateTerminal) {
        log.info("OE更新终端信息参数:{}", toUpdateTerminal);
        return R.ok();
    }

    /**
     * 药企分页查询
     *
     * @param page 分页参数
     * @param name 药企名称
     * @param code 统一社会信用代码
     * @return 分页结果
     */
    @GetMapping("/page-drug-company")
    public R<IPage<OeTenantVO>> pageDrugCompany(Page<OeTenantVO> page, String name, String code) {
        int total = 20;
        List<OeTenantVO> list = new ArrayList<>();
        for (int i = Math.toIntExact(page.getCurrent()); i <= page.getSize() && i <= total; i++) {
            OeTenantVO oeTenantVO = new OeTenantVO();
            oeTenantVO.setTenantId((long) i);
            oeTenantVO.setCode("123456789123456");
            oeTenantVO.setName("测试药企" + (i));
            oeTenantVO.setLevelName("药企");
            oeTenantVO.setPermissions(new String[]{"导出"});
            oeTenantVO.setSettleChannels(new String[]{"中智","正启之星"});
            oeTenantVO.setDelFlagName("启用");
            oeTenantVO.setCreatedTime(LocalDateTime.now());
            oeTenantVO.setModifiedTime(LocalDateTime.now());
            oeTenantVO.setDelFlag("OK");
            oeTenantVO.setDelFlagName("正常");
            oeTenantVO.setLocFlag("OK");
            oeTenantVO.setLocFlagName("正常");
            list.add(oeTenantVO);
        }

        page.setRecords(list);
        page.setTotal(total);
        page.setPages((total + page.getSize() - 1) / page.getSize());

        return R.ok(page);
    }

    /**
     * 药企查询
     *
     * @return {@link R}<{@link List}<{@link Map}<{@link String}, {@link String}>>>
     */
    @GetMapping("/list-drug-company")
    public R<List<Map<String, Object>>> listDrugCompany() {
        int total = 20;
        List<Map<String, Object>> list = new ArrayList<>();
        for (int i = 0; i < total; i++) {
            Map<String, Object> map = new HashMap<>();
            map.put("name", "测试药企" + (i));
            map.put("tenantId", (long) i);
            list.add(map);
        }

        return R.ok(list);
    }

    /**
     * CSO分页查询
     *
     * @param page 分页参数
     * @param name 药企名称
     * @param code 统一社会信用代码
     * @return 分页结果
     */
    @GetMapping("/page-cso")
    public R<IPage<OeTenantVO>> pageCso(Page<OeTenantVO> page, String name, String code) {
        int total = 20;
        List<OeTenantVO> list = new ArrayList<>();
        for (int i = Math.toIntExact(page.getCurrent()); i <= page.getSize() && i <= total; i++) {
            OeTenantVO oeTenantVO = new OeTenantVO();
            oeTenantVO.setTenantId((long) i);
            oeTenantVO.setCode("123456789123456");
            oeTenantVO.setName("测试代理商" + (i));
            oeTenantVO.setLevelName("CSO");
            oeTenantVO.setPermissions(new String[]{"导出"});
            oeTenantVO.setSettleChannels(new String[]{"中智","正启之星"});
            oeTenantVO.setDelFlagName("启用");
            oeTenantVO.setCreatedTime(LocalDateTime.now());
            oeTenantVO.setModifiedTime(LocalDateTime.now());
            oeTenantVO.setDelFlag("OK");
            oeTenantVO.setDelFlagName("正常");
            oeTenantVO.setLocFlag("OK");
            oeTenantVO.setLocFlagName("正常");
            list.add(oeTenantVO);
        }

        page.setRecords(list);
        page.setTotal(total);
        page.setPages((total + page.getSize() - 1) / page.getSize());

        return R.ok(page);
    }

    /**
     * CSO分页查询-用于药企
     *
     * @param page 分页参数
     * @param name 药企名称
     * @param code 统一社会信用代码
     * @return 分页结果
     */
    @GetMapping("/page-cso-for-dept")
    public R<IPage<OeTenantVO>> pageCsoForDept(Page<OeTenantVO> page, String name, String code, String drugCompanyName) {
        // 企业的权限 TODO

        int total = 10;
        List<OeTenantVO> list = new ArrayList<>();
        for (int i = Math.toIntExact(page.getCurrent()); i <= page.getSize() && i <= total; i++) {
            OeTenantVO oeTenantVO = new OeTenantVO();
            oeTenantVO.setTenantId((long) i);
            oeTenantVO.setCode("123456789123456");
            oeTenantVO.setName("测试代理商" + (i));
            oeTenantVO.setLevelName("CSO");
            oeTenantVO.setPermissions(new String[]{"导出"});
            oeTenantVO.setSettleChannels(new String[]{"中智","正启之星"});
            oeTenantVO.setDelFlagName("启用");
            oeTenantVO.setCreatedTime(LocalDateTime.now());
            oeTenantVO.setModifiedTime(LocalDateTime.now());
            oeTenantVO.setDelFlag("OK");
            oeTenantVO.setDelFlagName("正常");
            oeTenantVO.setLocFlag("OK");
            oeTenantVO.setLocFlagName("正常");
            list.add(oeTenantVO);
        }

        page.setRecords(list);
        page.setTotal(total);
        page.setPages((total + page.getSize() - 1) / page.getSize());

        return R.ok(page);
    }

    /**
     * 终端分页查询(平台)
     *
     * @param page 分页参数
     * @param name 药企名称
     * @param code 统一社会信用代码
     * @return 分页结果
     */
    @GetMapping("/page-terminal")
    public R<IPage<OeTenantVO>> pageTerminal(Page<OeTenantVO> page, String name, String code) {
        int total = 20;
        List<OeTenantVO> list = new ArrayList<>();
        for (int i = Math.toIntExact(page.getCurrent()); i <= page.getSize() && i <= total; i++) {
            OeTenantVO oeTenantVO = new OeTenantVO();
            oeTenantVO.setTenantId((long) i);
            oeTenantVO.setCode("123456789123456");
            oeTenantVO.setName("测试终端" + (i));
            oeTenantVO.setLevelName("CSO");
            oeTenantVO.setPermissions(new String[]{"导出"});
            oeTenantVO.setSettleChannels(new String[]{"中智","正启之星"});
            oeTenantVO.setDelFlagName("启用");
            oeTenantVO.setCreatedTime(LocalDateTime.now());
            oeTenantVO.setModifiedTime(LocalDateTime.now());
            oeTenantVO.setDelFlag("OK");
            oeTenantVO.setDelFlagName("正常");
            oeTenantVO.setLocFlag("OK");
            oeTenantVO.setLocFlagName("正常");
            list.add(oeTenantVO);
        }

        page.setRecords(list);
        page.setTotal(total);
        page.setPages((total + page.getSize() - 1) / page.getSize());

        return R.ok(page);
    }

    /**
     * 终端分页查询(药企)
     *
     * @param page 分页参数
     * @param name 药企名称
     * @param code 统一社会信用代码
     * @return 分页结果
     */
    @GetMapping("/page-terminal-for-dept")
    public R<IPage<OeTerminalVO>> pageTerminalForDept(Page<OeTerminalVO> page, String name, String code) {
        int total = 10;
        List<OeTerminalVO> list = new ArrayList<>();
        for (int i = Math.toIntExact(page.getCurrent()); i <= page.getSize() && i <= total; i++) {
            OeTerminalVO vo = new OeTerminalVO();
            vo.setName(name);
            vo.setCode("123456789123456");
            vo.setTenantId((long) i);
            vo.setServedCsoTenantId(12312L);
            vo.setServedCsoName("代理商123");
            vo.setServedDrugCompanyTenantId(23423L);
            vo.setServedDrugCompanyName("药企234");
            vo.setDrugCompanyApprovalStatus(i/2==0 ? "REJECT" : "PASS");
            vo.setDrugCompanyApprovalStatusName(i/2==0 ? "审核不通过" : "通过");
            vo.setCsoApprovalStatus("PASS");
            vo.setCsoApprovalStatusName("审核通过");
            list.add(vo);
        }

        page.setRecords(list);
        page.setTotal(total);
        page.setPages((total + page.getSize() - 1) / page.getSize());

        return R.ok(page);
    }
}
