package com.qunzhixinxi.hnqz.admin.oe.entity.dto;

import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;


/**
 * oe租户请求
 *
 * @author lixuesong
 * @date 2023/08/04
 */
public class OeTenantRequest {

    /**
     * 保存药企信息
     *
     * @author lixuesong
     * @date 2023/08/04
     */
    @Data
    public static class ToSaveDrugCompany implements Serializable {
        private static final long serialVersionUID = -2731914147485157197L;

        /**
         * 企业名称
         */
        private String name;

        /**
         * 统一社会信用代码
         */
        private String code;

        /**
         * 权限限制
         */
        private String[] permissions;

        /**
         * 锁定标记(OK-正常,LOC-已锁定)
         */
        private String locFlag;
    }

    /**
     * 更新药企信息
     *
     * @author lixuesong
     * @date 2023/08/04
     */
    @Data
    public static class ToUpdateDrugCompany implements Serializable {
        private static final long serialVersionUID = -8743428178584985176L;

        /**
         * 租户id
         */
        private Long tenantId;

        /**
         * 企业名称
         */
        private String name;

        /**
         * 统一社会信用代码
         */
        private String code;

        /**
         * 权限限制
         */
        private String[] permissions;

        /**
         * 锁定标记
         */
        private String locFlag;
    }

    /**
     * 启用/停用药企
     *
     * @author lixuesong
     * @date 2023/08/04
     */
    @Data
    public static class ToLockUnlock implements Serializable {

        private static final long serialVersionUID = -5033454666284018069L;

        /**
         * 租户id
         */
        @NotNull(message = "主键ID必填")
        private Long tenantId;

        /**
         * 锁定标记(OK-正常,LOC-已锁定)
         */
        @NotBlank(message = "锁定标记必填")
        private String locFlag;
    }

    /**
     * 保存代理商
     *
     * @author lixuesong
     * @date 2023/08/04
     */
    @Data
    public static class ToSaveCso implements Serializable {
        private static final long serialVersionUID = -3930419806595107388L;
        
        /**
         * 企业名称
         */
        private String name;

        /**
         * 统一社会信用代码/身份证号
         */
        private String code;

        /**
         * 代理商类型
         */
        private String csoType;

        /**
         * 推广药品
         */
        private Long medicineId;

        /**
         * 营业执照
         */
        private String[] licenseUrl;

        /**
         * 社保缴纳证明
         */
        private String[] socialSecurityUrl;

        /**
         * 身份证正面url
         */
        private String idCardFrontUrl;

        /**
         * 身份证反面url
         */
        private String idCardBackUrl;

        /**
         * 身份证手持url
         */
        private String idCardHoldingUrl;

        /**
         * 锁定标记(OK-正常,LOC-已锁定)
         */
        private String locFlag;
    }

    /**
     * 更新代理商
     *
     * @author lixuesong
     * @date 2023/08/04
     */
    @Data
    public static class ToUpdateCso implements Serializable {
        private static final long serialVersionUID = 6561938432965789032L;

        /**
         * 租户id
         */
        private Long tenantId;

        /**
         * 企业名称
         */
        private String name;

        /**
         * 统一社会信用代码/身份证号
         */
        private String code;

        /**
         * 代理商类型
         */
        private String csoType;

        /**
         * 推广药品
         */
        private Long medicineId;

        /**
         * 营业执照
         */
        private String[] licenseUrl;

        /**
         * 社保缴纳证明
         */
        private String[] socialSecurityUrl;

        /**
         * 身份证正面url
         */
        private String idCardFrontUrl;

        /**
         * 身份证反面url
         */
        private String idCardBackUrl;

        /**
         * 身份证手持url
         */
        private String idCardHoldingUrl;

        /**
         * 锁定标记(OK-正常,LOC-已锁定)
         */
        private String locFlag;
    }

    /**
     * 保存终端参数
     *
     * @author lixuesong
     * @date 2023/08/04
     */
    @Data
    public static class ToSaveTerminal implements Serializable {
        private static final long serialVersionUID = 3015462481533357880L;

        /**
         * 企业名称
         */
        @NotBlank(message = "企业名称必填")
        private String name;

        /**
         * 统一社会信用代码
         */
        @NotBlank(message = "统一社会信用代码必填")
        private String code;

        /**
         * 推广药品
         */
        private Long medicineId;

        /**
         * 营业执照
         */
        private String[] licenseUrl;

        /**
         * 社保缴纳证明
         */
        private String[] socialSecurityUrl;

        /**
         * 锁定标记(OK-正常,LOC-已锁定)
         */
        private String locFlag;
    }

    /**
     * 保存终端参数
     *
     * @author lixuesong
     * @date 2023/08/04
     */
    @Data
    public static class ToUpdateTerminal implements Serializable {
        private static final long serialVersionUID = -1539062519797105804L;

        /**
         * 企业名称
         */
        @NotBlank(message = "企业名称必填")
        private String name;

        /**
         * 统一社会信用代码
         */
        @NotBlank(message = "统一社会信用代码必填")
        private String code;

        /**
         * 推广药品
         */
        private Long medicineId;

        /**
         * 营业执照
         */
        private String[] licenseUrl;

        /**
         * 社保缴纳证明
         */
        private String[] socialSecurityUrl;

        /**
         * 锁定标记(OK-正常,LOC-已锁定)
         */
        private String locFlag;
    }
}
