/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */
package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.csp.sentinel.util.StringUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.admin.api.dto.UserDTO;
import com.qunzhixinxi.hnqz.admin.entity.WmDaPharmacy;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackageStatus;
import com.qunzhixinxi.hnqz.admin.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.entity.WmTaskContent;
import com.qunzhixinxi.hnqz.admin.entity.model.excel.BatchPharmacyExcelModel;
import com.qunzhixinxi.hnqz.admin.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.mapper.WmDaPharmacyMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmTaskContentMapper;
import com.qunzhixinxi.hnqz.admin.service.WmDaPharmacyService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageStatusService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskService;
import com.qunzhixinxi.hnqz.admin.util.MapUtil;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 档案-药店
 *
 * @author pigx code generator
 * @date 2020-09-23 21:17:34
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmDaPharmacyServiceImpl extends ServiceImpl<WmDaPharmacyMapper, WmDaPharmacy> implements WmDaPharmacyService {


	private final WmDaPharmacyMapper wmDaPharmacyMapper;
	private final WmTaskService wmTaskService;
	private final WmTaskContentMapper wmTaskContentMapper;
	private final WmScorePackageService wmScorePackageService;
	private final WmScorePackageStatusService packageStatusService;

	@Override
	public R approvalPharmacy(String taskId) {
		WmDaPharmacy wmDaPharmacy=new WmDaPharmacy();
		//获取任务信息
		WmTask wmTask=wmTaskService.getById(taskId);
		if(null==wmTask){
			return R.failed("未查询到调研信息");
		}
		// if("1".equals(wmTask.getPlatAuditStatus())){
		// 	return R.failed("信息已审核，请重新查询后操作");
		// }
		WmTaskContent wmTaskContent = wmTaskContentMapper.selectById(wmTask.getTaskContentId());
		if(null==wmTaskContent){
			return R.failed("调研信息为空");
		}
		// 根据省市区+详情地址+药店名称查询药店
		WmDaPharmacy query = new WmDaPharmacy();
		// 省市区
		if (StringUtil.isNotBlank(wmTaskContent.getTemp7())) {
			if (StrUtil.contains(wmTaskContent.getTemp7(), "-")) {
				String[] citys = wmTaskContent.getTemp7().split("-");
				query.setProvince(ArrayUtil.get(citys, 0));
				query.setCity(ArrayUtil.get(citys, 1));
				query.setArea(ArrayUtil.get(citys, 2));
			} else if (StrUtil.contains(wmTaskContent.getTemp7(), ",")) {
				String[] citys = wmTaskContent.getTemp7().split(",");
				query.setProvince(ArrayUtil.get(citys, 0));
				query.setCity(ArrayUtil.get(citys, 1));
				query.setArea(ArrayUtil.get(citys, 2));
			}
		}
		// 详细地址
		query.setAddress(wmTaskContent.getTemp4());
		// 药店名称
		query.setPharmacyName(wmTaskContent.getTemp3());
		query.setDelFlag("0");
		List<WmDaPharmacy> wdplist = wmDaPharmacyMapper.selectList(Wrappers.query(query));
		if(CollectionUtil.isNotEmpty(wdplist)) {
			wmDaPharmacy = wdplist.get(0);
			wmTask.setCompareResult("2");//信息不一致
		} else {
			wmTask.setCompareResult("1");//新增
		}

		// 获取经纬度
		this.convertToLatAndLong(query);
		wmDaPharmacy.setLatitude(query.getLatitude());
		wmDaPharmacy.setLongitude(query.getLongitude());


		Map<String,Object> result = new HashMap();
		result.put("info", wmDaPharmacy);//数据
		result.put("_wmTask", wmTask);
		result.put("wmTask", wmTaskContent);
		result.put("wmTaskContent", wmTaskContent);
		result.put("compareResult", wmTask.getCompareResult());//数据
		return R.ok(result);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public R updateByapprovalForManualOperation(Map<String, String> map) {
		String taskId=map.get("taskId");
		WmTask wmTask=wmTaskService.getById(taskId);
		if(wmTask==null){
			return R.failed("未查询到调查信息");
		}
		String approvalInfo=map.get("approvalInfo");
		String approvalOpinion=map.get("approvalOpinion");
		String compareResult=map.get("compareResult");
		//更新时，同一个task_content_id和createtime下的数据，都要更新状态为已审核
//		LambdaUpdateWrapper<WmTask> updateWrapper=new LambdaUpdateWrapper<WmTask>();
//		updateWrapper.set(WmTask::getPlatAuditStatus,"1");
//		updateWrapper.set(WmTask::getApprovalOpinion,approvalOpinion);
//		updateWrapper.set(WmTask::getCompareResult,compareResult);
//		updateWrapper.set(WmTask::getApprovalInfo,approvalInfo);
//		updateWrapper.eq(WmTask::getTaskContentId,wmTask.getTaskContentId());
//		updateWrapper.eq(WmTask::getCreateTime,wmTask.getCreateTime());

//		WmTask updateTask = new WmTask();
//		updateTask.setId(taskId);
//		updateTask.setPlatAuditStatus("1");
//		updateTask.setApprovalOpinion(approvalOpinion);
//		updateTask.setCompareResult(compareResult);
//		updateTask.setApprovalInfo(approvalInfo);

		if("1".equals(approvalOpinion)){//采纳
			//updateWrapper.set(WmTask::getTaskStatus,"3");//审核通过
			// TODO 根据任务积分修改积分包状态
//			updateWrapper.set(WmTask::getRealFlag,"0");//0审核通过
//			updateTask.setRealFlag("0");

			WmDaPharmacy wmDaPharmacy = new WmDaPharmacy();
			WmTaskContent wmTaskContent = wmTaskContentMapper.selectById(wmTask.getTaskContentId());
			//更新审核信息
			LocalDateTime now = LocalDateTime.now();
//			Integer userId = SecurityUtils.getUser().getId();
			if("1".equals(compareResult)){//新增
				//新增
				wmDaPharmacy.setPharmacyName(wmTaskContent.getTemp3());//名称
				wmDaPharmacy.setAddress(wmTaskContent.getTemp4());//名称
				if (!StringUtil.isEmpty(wmTaskContent.getTemp7())) {//省市区
					String[] citys = wmTaskContent.getTemp7().split("-");
					if(citys.length==3) {
						wmDaPharmacy.setProvince(citys[0]);
						wmDaPharmacy.setCity(citys[1]);
						wmDaPharmacy.setArea(citys[2]);
					}else {
						String[] citys1=wmTaskContent.getTemp7().split(",");
						if(citys1.length==3) {
							wmDaPharmacy.setProvince(citys1[0]);
							wmDaPharmacy.setCity(citys1[1]);
							wmDaPharmacy.setArea(citys1[2]);
						}
					}
				}
				wmDaPharmacy.setCreateTime(now);
				wmDaPharmacy.setCreateUser(0);
				// 根据地址转换经纬度
				this.convertToLatAndLong(wmDaPharmacy);
				this.save(wmDaPharmacy);
			}else if("2".equals(compareResult)){//信息不一致
				//对比药店和调研信息,只有所属地区不一样
//				List<WmDaPharmacy> wdplist=wmDaPharmacyMapper.slectByParam(wmTaskContent.getTemp3());
//				if(null!=wdplist&&wdplist.size()>0){
//					wmDaPharmacy=wdplist.get(0);
//				}
//				String address=wmDaPharmacy.getProvince()+"-"+wmDaPharmacy.getCity()+"-"+wmDaPharmacy.getArea();
//				if(null!=wmTaskContent.getTemp7()&&!wmTaskContent.getTemp7().equals(address)){
//					String[] citys=wmTaskContent.getTemp7().split("-");
//					if(citys.length==3){
//						wmDaPharmacy.setProvince(citys[0]);
//						wmDaPharmacy.setCity(citys[1]);
//						wmDaPharmacy.setArea(citys[2]);
//					}else {
//						String[] citys1=wmTaskContent.getTemp7().split(",");
//						if(citys1.length==3) {
//							wmDaPharmacy.setProvince(citys1[0]);
//							wmDaPharmacy.setCity(citys1[1]);
//							wmDaPharmacy.setArea(citys1[2]);
//						}
//					}
//				}
//				wmDaPharmacy.setId(map.get("infoId"));
//				wmDaPharmacy.setPharmacyName(wmTaskContent.getTemp3());//名称
//				wmDaPharmacy.setAddress(wmTaskContent.getTemp4());
//				wmDaPharmacy.setUpdateTime(now);
//				wmDaPharmacy.setUpdateUser(userId);
				// 根据地址转换经纬度
//				this.convertToLatAndLong(wmDaPharmacy);
				//更新药店信息;
//				this.updateById(wmDaPharmacy);
			}

		}else{
			//updateWrapper.set(WmTask::getTaskStatus,"2");//审核不通过
//			updateWrapper.set(WmTask::getRealFlag,"1");//1不通过

//			updateTask.setRealFlag("1");
		}
		//提交
//		updateTask.setTaskStatus("2");
		WmScorePackage wmScorePackage = wmScorePackageService.getById(wmTask.getScorePackageId());
// 		if("1".equals(approvalOpinion)){
// 			if("1".equals(wmScorePackage.getPackageType1())){
// 				Boolean fullFlag = wmTaskService.checkUserTaskSubScore(wmTask.getTaskUserId());
// 				if(fullFlag){
// //					return R.failed(1, "每日最多提交5万积分");
// 					updateTask.setScore(0);
// 				}
//
// 			}
// 		}
//		wmTaskService.updateById(updateTask);
//		wmTaskService.update(updateWrapper);

//		if("1".equals(approvalOpinion)){
//
//		}else {

			/**
			 * 审核不通过可能需要释放积分
			 */
			//更新积分包状态
//			WmScorePackage wmScorePackage = wmScorePackageService.getById(wmTask.getScorePackageId());
			WmScorePackageStatus query = new WmScorePackageStatus();
			query.setUserId(wmTask.getTaskUserId());
			query.setPackageId(wmTask.getScorePackageId());
			query.setStatus("2");
			WmScorePackageStatus scorePackageStatus = packageStatusService.getOne(Wrappers.query(query));

			/**
			 * 不需要释放 直接返回
			 */
//			if("1".equals(wmScorePackage.getTaskAddFlag()) && "1".equals(scorePackageStatus.getTaskAddFlag())){
//				return R.ok();
//			}

//			packageStatusService.updatePackageAndStatus(wmTask, wmScorePackage, scorePackageStatus);
//		}
		return R.ok();
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public R updateByapproval(Map<String, String> map) {
		String taskId=map.get("taskId");
		WmTask wmTask=wmTaskService.getById(taskId);
		if(wmTask==null){
			return R.failed("未查询到调查信息");
		}
		String approvalInfo=map.get("approvalInfo");
		String approvalOpinion=map.get("approvalOpinion");
		String compareResult=map.get("compareResult");
		//更新时，同一个task_content_id和createtime下的数据，都要更新状态为已审核
//		LambdaUpdateWrapper<WmTask> updateWrapper=new LambdaUpdateWrapper<WmTask>();
//		updateWrapper.set(WmTask::getPlatAuditStatus,"1");
//		updateWrapper.set(WmTask::getApprovalOpinion,approvalOpinion);
//		updateWrapper.set(WmTask::getCompareResult,compareResult);
//		updateWrapper.set(WmTask::getApprovalInfo,approvalInfo);
//		updateWrapper.eq(WmTask::getTaskContentId,wmTask.getTaskContentId());
//		updateWrapper.eq(WmTask::getCreateTime,wmTask.getCreateTime());

//		WmTask updateTask = new WmTask();
//		updateTask.setId(taskId);
//		updateTask.setPlatAuditStatus("1");
//		updateTask.setApprovalOpinion(approvalOpinion);
//		updateTask.setCompareResult(compareResult);
//		updateTask.setApprovalInfo(approvalInfo);

		if("1".equals(approvalOpinion)){//采纳
			//updateWrapper.set(WmTask::getTaskStatus,"3");//审核通过
			// TODO 根据任务积分修改积分包状态
//			updateWrapper.set(WmTask::getRealFlag,"0");//0审核通过
//			updateTask.setRealFlag("0");

			WmDaPharmacy wmDaPharmacy = new WmDaPharmacy();
			WmTaskContent wmTaskContent = wmTaskContentMapper.selectById(wmTask.getTaskContentId());
			//更新审核信息
			LocalDateTime now = LocalDateTime.now();
			Integer userId = SecurityUtils.getUser().getId();
			if("1".equals(compareResult)){//新增
				//新增
				wmDaPharmacy.setPharmacyName(wmTaskContent.getTemp3());//名称
				wmDaPharmacy.setAddress(wmTaskContent.getTemp4());//名称
				if (!StringUtil.isEmpty(wmTaskContent.getTemp7())) {//省市区
					String[] citys = wmTaskContent.getTemp7().split("-");
					if(citys.length==3) {
						wmDaPharmacy.setProvince(citys[0]);
						wmDaPharmacy.setCity(citys[1]);
						wmDaPharmacy.setArea(citys[2]);
					}else {
						String[] citys1=wmTaskContent.getTemp7().split(",");
						if(citys1.length==3) {
							wmDaPharmacy.setProvince(citys1[0]);
							wmDaPharmacy.setCity(citys1[1]);
							wmDaPharmacy.setArea(citys1[2]);
						}
					}
				}
				wmDaPharmacy.setCreateTime(now);
				wmDaPharmacy.setCreateUser(userId);
				// 根据地址转换经纬度
				this.convertToLatAndLong(wmDaPharmacy);
				this.save(wmDaPharmacy);
			}else if("2".equals(compareResult)){//信息不一致
				//对比药店和调研信息,只有所属地区不一样
//				List<WmDaPharmacy> wdplist=wmDaPharmacyMapper.slectByParam(wmTaskContent.getTemp3());
//				if(null!=wdplist&&wdplist.size()>0){
//					wmDaPharmacy=wdplist.get(0);
//				}
//				String address=wmDaPharmacy.getProvince()+"-"+wmDaPharmacy.getCity()+"-"+wmDaPharmacy.getArea();
//				if(null!=wmTaskContent.getTemp7()&&!wmTaskContent.getTemp7().equals(address)){
//					String[] citys=wmTaskContent.getTemp7().split("-");
//					if(citys.length==3){
//						wmDaPharmacy.setProvince(citys[0]);
//						wmDaPharmacy.setCity(citys[1]);
//						wmDaPharmacy.setArea(citys[2]);
//					}else {
//						String[] citys1=wmTaskContent.getTemp7().split(",");
//						if(citys1.length==3) {
//							wmDaPharmacy.setProvince(citys1[0]);
//							wmDaPharmacy.setCity(citys1[1]);
//							wmDaPharmacy.setArea(citys1[2]);
//						}
//					}
//				}
//				wmDaPharmacy.setId(map.get("infoId"));
//				wmDaPharmacy.setPharmacyName(wmTaskContent.getTemp3());//名称
//				wmDaPharmacy.setAddress(wmTaskContent.getTemp4());
//				wmDaPharmacy.setUpdateTime(now);
//				wmDaPharmacy.setUpdateUser(userId);
				// 根据地址转换经纬度
//				this.convertToLatAndLong(wmDaPharmacy);
				//更新药店信息;
//				this.updateById(wmDaPharmacy);
			}

		}else{
			//updateWrapper.set(WmTask::getTaskStatus,"2");//审核不通过
//			updateWrapper.set(WmTask::getRealFlag,"1");//1不通过

//			updateTask.setRealFlag("1");
		}
		//提交
//		updateTask.setTaskStatus("2");
//		WmScorePackage wmScorePackage = wmScorePackageService.getById(wmTask.getScorePackageId());
// 		if("1".equals(approvalOpinion)){
// 			if("1".equals(wmScorePackage.getPackageType1())){
// 				Boolean fullFlag = wmTaskService.checkUserTaskSubScore(wmTask.getTaskUserId());
// 				if(fullFlag){
// //					return R.failed(1, "每日最多提交5万积分");
// 					updateTask.setScore(0);
// 				}
//
// 			}
// 		}
//		wmTaskService.updateById(updateTask);
//		wmTaskService.update(updateWrapper);

//		if("1".equals(approvalOpinion)){
//
//		}else {

			/**
			 * 审核不通过可能需要释放积分
			 */
			//更新积分包状态
//			WmScorePackage wmScorePackage = wmScorePackageService.getById(wmTask.getScorePackageId());
//			WmScorePackageStatus query = new WmScorePackageStatus();
//			query.setUserId(wmTask.getTaskUserId());
//			query.setPackageId(wmTask.getScorePackageId());
//			query.setStatus("2");
//			WmScorePackageStatus scorePackageStatus = packageStatusService.getOne(Wrappers.query(query));

			/**
			 * 不需要释放 直接返回
			 */
//			if("1".equals(wmScorePackage.getTaskAddFlag()) && "1".equals(scorePackageStatus.getTaskAddFlag())){
//				return R.ok();
//			}

//			packageStatusService.updatePackageAndStatus(wmTask, wmScorePackage, scorePackageStatus);
//		}
		return R.ok();
	}

	/**
	 * 根据经纬度查询
	 *
	 * @param wmDaPharmacy
	 * @return
	 */
	@Override
	public List<WmDaPharmacy> listByPoint(WmDaPharmacy wmDaPharmacy) {
		return wmDaPharmacyMapper.listByPoint(wmDaPharmacy);
	}

	/**
	 * 批量转换经纬度
	 */
	@Override
	public void parseAddress() {
		List<WmDaPharmacy> list = this.list(Wrappers.<WmDaPharmacy>lambdaQuery().isNull(WmDaPharmacy::getLatitude));
		if (CollectionUtil.isNotEmpty(list)) {
			list.forEach(wmDaPharmacy -> {
				this.convertToLatAndLong(wmDaPharmacy);
				this.saveOrUpdate(wmDaPharmacy);
			});
		}
	}

	/**
	 * 批量导入
	 *
	 * @param excelModelList 药店列表
	 * @return 结果
	 */
	@Override
	public Map<String, String> batchByExcel(List<BatchPharmacyExcelModel> excelModelList) {

		// 变量
		Map<String, String> errorMap = new HashMap<>();
		List<WmDaPharmacy> currentList = new ArrayList<>();

		// 获取操作员
		HnqzUser creator = SecurityUtils.getUser();

		log.info("开始校验数据：{}", excelModelList);
		// 业务值校验
		Map<String, List<BatchPharmacyExcelModel>> nameMap = excelModelList.stream()
				.filter(excelModel -> StrUtil.isNotBlank(excelModel.getName()))
				.collect(Collectors.groupingBy(BatchPharmacyExcelModel::getName));

		for (String nameStr : nameMap.keySet()) {
			List<BatchPharmacyExcelModel> excelModels = nameMap.get(nameStr);
			if (excelModels.size() > 1) {
				log.error("名称重复");
				String idStr = excelModels.stream().map(BatchPharmacyExcelModel::getNumber).collect(Collectors.joining(","));
				errorMap(errorMap, "名称重复", idStr);
				continue;
			}

			BatchPharmacyExcelModel excelModel = excelModels.get(0);

			WmDaPharmacy pharmacy = new WmDaPharmacy();
			// 校验名称
			String name = StrUtil.cleanBlank(excelModel.getName());
			if (StrUtil.isBlank(name)) {
				log.error("名称为空");
				errorMap(errorMap, "名称为空", excelModel.getNumber());
			} else {
				pharmacy.setPharmacyName(name);
			}

			// 校验地址
			String address = StrUtil.cleanBlank(excelModel.getAddress());
			if (StrUtil.isBlank(address)) {
				log.error("地址为空");
				errorMap(errorMap, "地址为空", excelModel.getNumber());
			} else {
				pharmacy.setAddress(address);
			}

			// 药店类型设置默认值
			if (StrUtil.isNotBlank(excelModel.getPharmacyType())) {
				UpmsType.PharmacyType pharmacyType = UpmsType.PharmacyType.getByDescription(excelModel.getPharmacyType());
				pharmacy.setPharmacyType(pharmacyType);
			} else {
				pharmacy.setPharmacyType(UpmsType.PharmacyType.ORDINARY);
			}

			// 重复校验
			int count = this.count(Wrappers.<WmDaPharmacy>lambdaQuery()
					.eq(WmDaPharmacy::getPharmacyName, name)
					.eq(WmDaPharmacy::getDelFlag, DelEnum.NOT_DEL.val()));
			if (count > 0) {
				log.error("{} 已存在", name);
				errorMap(errorMap, "名称已存在", excelModel.getNumber());
			}

			// 记录正确的结果
			pharmacy.setProvince("");
			pharmacy.setCity("");
			pharmacy.setArea("");
			LocalDateTime now = LocalDateTime.now();
			pharmacy.setCreateTime(now);
			pharmacy.setCreateUser(creator.getId());
			// 根据地址转换经纬度
			this.convertToLatAndLong(pharmacy);
			currentList.add(pharmacy);
		}

		// 上传结果
		if (CollUtil.isEmpty(errorMap)) {
			this.saveBatch(currentList);
		}

		return errorMap;
	}

	/**
	 * 根据地址转换经纬度
	 *
	 * @param wmDaPharmacy
	 */
	private void convertToLatAndLong(WmDaPharmacy wmDaPharmacy) {
		String fullAddress = wmDaPharmacy.getProvince() + wmDaPharmacy.getCity() + wmDaPharmacy.getArea()
				+ wmDaPharmacy.getAddress();
		JSONObject jsonObject = MapUtil.getLatAndLogByName(fullAddress);
		if (jsonObject != null) {
			wmDaPharmacy.setProvince(jsonObject.getString("province"));
			wmDaPharmacy.setCity(jsonObject.getString("city"));
			wmDaPharmacy.setArea(jsonObject.getString("district"));
			String location = jsonObject.getString("location");
			if (null != location && location.split(",").length > 0) {
				wmDaPharmacy.setLongitude(location.split(",")[0]);
				wmDaPharmacy.setLatitude(location.split(",")[1]);
			}
		}
	}

	/**
	 * 封装错误信息
	 *
	 * @param errorMap 记录错误信息map
	 * @param key      原因
	 * @param idStr    错误id
	 */
	private void errorMap(Map<String, String> errorMap, String key, String idStr) {
		if (errorMap.containsKey(key)) {
			String val = errorMap.get(key);
			idStr = String.join(";", val, idStr);
		}
		errorMap.put(key, idStr);
	}

}
