package com.qunzhixinxi.hnqz.admin;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskSubmissionPercentRule;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskSubmissionRule;
import com.qunzhixinxi.hnqz.admin.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.mapper.WmTaskMapper;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskSubmissionPercentRuleService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskSubmissionRuleService;
import com.qunzhixinxi.hnqz.common.core.constant.enums.CommonFlag;
import lombok.extern.slf4j.Slf4j;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author snows
 * @date 2023/7/18 17:00
 */
@Slf4j
@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest(classes = {HnqzAdminApplication.class, Test.class})
public class WmTaskSubmissionRuleServiceTest {

    @Autowired
    private WmTaskSubmissionPercentRuleService taskSubmissionPercentRuleService;
	@Autowired
	private WmTaskSubmissionRuleService taskSubmissionRuleService;
    @Autowired
    private SysDeptService deptService;
	@Autowired
	private WmTaskMapper wmTaskMapper;

	/**
	 * 初始化企业的特定任务规则
	 */
//	@Test
	public void initDeptTaskSubmissionRule() {
		// 查询源数据企业的配置
		List<WmTaskSubmissionRule> platformRules = taskSubmissionRuleService.list(Wrappers.<WmTaskSubmissionRule>lambdaQuery()
				.eq(WmTaskSubmissionRule::getDeptId, 1)
				.eq(WmTaskSubmissionRule::getSubCategory, UpmsType.TaskSubCategory.PHARMACY_VISIT));

		// 查询需要初始化的企业
		List<SysDept> deptList = deptService.list(Wrappers.<SysDept>lambdaQuery()
				.eq(SysDept::getDelFlag, DelEnum.NOT_DEL.val())
				.ne(SysDept::getDeptId, 1));
		Set<Integer> deptIds = deptList.stream().map(SysDept::getDeptId).collect(Collectors.toSet());

		List<WmTaskSubmissionRule> deptRules = new ArrayList<>(deptIds.size() * platformRules.size());

		deptIds.forEach(deptId -> {
			platformRules.forEach(r -> {
				WmTaskSubmissionRule deptRule = new WmTaskSubmissionRule();
				deptRule.setRule(r.getRule());
				deptRule.setDeptId(deptId);
				deptRule.setOptFlag(r.getOptFlag());
				deptRule.setSubCategory(r.getSubCategory());
				deptRules.add(deptRule);
			});

		});

		taskSubmissionRuleService.saveBatch(deptRules);
	}


//    @Test
    public void initTaskTypeConfig() {

        // 查询所有企业
        List<SysDept> depts = deptService.list();

        // 查询平台企业的规则
        List<WmTaskSubmissionPercentRule> baseRules = taskSubmissionPercentRuleService.list(Wrappers.<WmTaskSubmissionPercentRule>lambdaQuery()
                .eq(WmTaskSubmissionPercentRule::getDeptId, 1)
                .eq(WmTaskSubmissionPercentRule::getSubCategory, UpmsType.TaskSubCategory2.TASK_TYPE_CONFIG));
        WmTaskSubmissionPercentRule.PercentRule rule = baseRules.get(0).getRule();


        depts.forEach(sysDept -> {
            // 查询待修改企业的规则
            List<WmTaskSubmissionPercentRule> queryRules = taskSubmissionPercentRuleService.list(Wrappers.<WmTaskSubmissionPercentRule>lambdaQuery()
                    .eq(WmTaskSubmissionPercentRule::getDeptId, sysDept.getDeptId())
                    .eq(WmTaskSubmissionPercentRule::getSubCategory, UpmsType.TaskSubCategory2.TASK_TYPE_CONFIG));
            if (CollUtil.isEmpty(queryRules)) {
                log.info("初始化deptId={}", sysDept.getDeptId());
                WmTaskSubmissionPercentRule percentRule = new WmTaskSubmissionPercentRule();
                percentRule.setDeptId(sysDept.getDeptId());
                percentRule.setRule(rule);
                percentRule.setSubCategory(UpmsType.TaskSubCategory2.TASK_TYPE_CONFIG);
                percentRule.setOptFlag(CommonFlag.OptFlag.OK);
                taskSubmissionPercentRuleService.save(percentRule);
            }
        });
    }

    /**
     * 增量更新企业的任务类型配置数据
     */
    @Test
    public void initNewTaskTypeConfig() {

        // 平台新增的任务类型
        Set<String> taskTypeIds = new HashSet<>();
        taskTypeIds.add("73");

            // 查询待修改企业的规则
            List<WmTaskSubmissionPercentRule> queryRules = taskSubmissionPercentRuleService.list(Wrappers.<WmTaskSubmissionPercentRule>lambdaQuery()
                    .eq(WmTaskSubmissionPercentRule::getSubCategory, UpmsType.TaskSubCategory2.TASK_TYPE_CONFIG));
            if (CollUtil.isNotEmpty(queryRules)) {
                queryRules.forEach(percentRule -> {
                    log.info("初始化deptId={}", percentRule.getDeptId());

                    String[] oldTaskTypeIds = percentRule.getRule().getTaskTypeIds();

                    WmTaskSubmissionPercentRule updateRule = new WmTaskSubmissionPercentRule();
                    updateRule.setRuleId(percentRule.getRuleId());
                    Set<String> newTaskTypeIdSet = Stream.of(oldTaskTypeIds).collect(Collectors.toSet());
                    newTaskTypeIdSet.addAll(taskTypeIds);
                    WmTaskSubmissionPercentRule.PercentRule updatePercentRule = BeanUtil.copyProperties(percentRule.getRule(), WmTaskSubmissionPercentRule.PercentRule.class);
                    updatePercentRule.setTaskTypeIds(newTaskTypeIdSet.stream().sorted(Comparator.comparingInt(Integer::parseInt)).toArray(String[]::new));
                    updateRule.setRule(updatePercentRule);
                    updateRule.setUpdateTime(LocalDateTime.now());
                    taskSubmissionPercentRuleService.updateById(updateRule);
                });
            }
    }

	/**
	 * 月度积分限制校验
	 */
//	@Test
	public void checkMonthScoreLimit() {
		Integer entId = 1611890565;
		// 月度积分限制
		String monthLimitValue;
		List<WmTaskSubmissionRule> monthScoreLimitRuleList = taskSubmissionRuleService.list(Wrappers.<WmTaskSubmissionRule>lambdaQuery()
				.in(WmTaskSubmissionRule::getDeptId, CollUtil.newHashSet("entId", 1))
				.eq(WmTaskSubmissionRule::getOptFlag, CommonFlag.OptFlag.OK)
				.eq(WmTaskSubmissionRule::getSubCategory, UpmsType.TaskSubCategory.MONTH_SCORE_LIMIT));
		if (CollUtil.isNotEmpty(monthScoreLimitRuleList)) {
			Map<Integer, WmTaskSubmissionRule.SubmissionRule> submissionRuleMap = monthScoreLimitRuleList.stream()
					.collect(Collectors.toMap(WmTaskSubmissionRule::getDeptId, rules ->
							BeanUtil.toBean(rules.getRule().get(0), WmTaskSubmissionRule.SubmissionRule.class)));
			WmTaskSubmissionRule.SubmissionRule submissionRule = submissionRuleMap.containsKey(entId) ? submissionRuleMap.get(entId) : submissionRuleMap.get(1);
			monthLimitValue = String.valueOf(submissionRule.getFrequency().get(0).get("quantity"));
		} else {
			monthLimitValue = "300000";
		}

		LocalDate localDate = LocalDate.now();
		LocalDateTime startOfMonth = localDate.withDayOfMonth(1).atStartOfDay();
		LocalDateTime startOfNextMonth = localDate.plusMonths(1).withDayOfMonth(1).atStartOfDay();
		int total = wmTaskMapper.getUserTaskSubScore("4714", startOfMonth, startOfNextMonth);

		System.out.println("total=" + total + " limit=" + monthLimitValue);
	}
}
