package com.qunzhixinxi.hnqz.daemon.quartz.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.SingleEnterpriseSettleSameAmountForSeveralMembersInCurrentMonth;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.SingleMemberSettleAmountOverrunInSeveralMonths;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.SingleMemberSettleSameAmountInSeveralMonths;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.SingleMemberWithoutSuchTaskTypesInSeveralMonths;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.SysJobLog;
import com.qunzhixinxi.hnqz.daemon.quartz.mapper.SingleEnterpriseSettleSameAmountForSeveralMembersInCurrentMonthMapper;
import com.qunzhixinxi.hnqz.daemon.quartz.mapper.SingleMemberSettleAmountOverrunInSeveralMonthsMapper;
import com.qunzhixinxi.hnqz.daemon.quartz.mapper.SingleMemberSettleSameAmountInSeveralMonthsMapper;
import com.qunzhixinxi.hnqz.daemon.quartz.mapper.SingleMemberWithoutSuchTaskTypesInSeveralMonthsMapper;
import com.qunzhixinxi.hnqz.daemon.quartz.service.SysJobLogService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;

/**
 * @author frwcloud
 * <p>
 * 定时任务执行日志表
 */
@RestController
@AllArgsConstructor
@RequestMapping("/sys-job-log")
public class SysJobLogController {

	private final SysJobLogService sysJobLogService;
	private final SingleMemberSettleSameAmountInSeveralMonthsMapper singleMemberSettleSameAmountInSeveralMonthsMapper;
	private final SingleMemberSettleAmountOverrunInSeveralMonthsMapper singleMemberSettleAmountOverrunInSeveralMonthsMapper;
	private final SingleEnterpriseSettleSameAmountForSeveralMembersInCurrentMonthMapper singleEnterpriseSettleSameAmountForSeveralMembersInCurrentMonthMapper;
	private final SingleMemberWithoutSuchTaskTypesInSeveralMonthsMapper singleMemberWithoutSuchTaskTypesInSeveralMonthsMapper;

	private final String j27 = "同一用户，在同一企业，连续<font color='red'>%s</font>月结算金额相同";
	private final String j28 = "同一企业，同一月份，给<font color='red'>%s</font>个及以上用户支付相同金额";
	private final String j29 = "同一用户，连续<font color='red'>%s</font>月没有定位打卡";
	private final String j30 = "同一用户，连续<font color='red'>%s</font>月结算金额超过<font color='red'>%s</font>金额";

	/**
	 * 分页查询
	 *
	 * @param page      分页对象
	 * @param sysJobLog 定时任务执行日志表
	 * @return
	 */
	@GetMapping("/page")
	public R getSysJobLogPage(Page<SysJobLog> page, SysJobLog sysJobLog) {

		sysJobLogService.page(page, Wrappers.<SysJobLog>lambdaQuery()
				.in(SysJobLog::getJobId, 27, 28, 29, 30)
				.like(StrUtil.isNotBlank(sysJobLog.getJobName()), SysJobLog::getJobName, sysJobLog.getJobName())
				.eq(StrUtil.isNotBlank(sysJobLog.getJobLogStatus()), SysJobLog::getJobLogStatus, sysJobLog.getJobLogStatus())
				.between(sysJobLog.getStartTime() != null && sysJobLog.getEndTime() != null, SysJobLog::getCreateTime, sysJobLog.getStartTime(), sysJobLog.getEndTime())
				.orderByDesc(SysJobLog::getCreateTime)

		);

		page.getRecords().forEach(r -> {

			// 匹配
			String jobName;
			int count = 0;
			if (r.getJobId() == 27) {
				jobName = String.format(j27, r.getMethodParamsValue());
				count = singleMemberSettleSameAmountInSeveralMonthsMapper.selectCount(Wrappers.<SingleMemberSettleSameAmountInSeveralMonths>lambdaQuery().eq(SingleMemberSettleSameAmountInSeveralMonths::getUuid, r.getUuid()));
			} else if (r.getJobId() == 28) {
				jobName = String.format(j28, r.getMethodParamsValue());
				count = singleEnterpriseSettleSameAmountForSeveralMembersInCurrentMonthMapper.selectCount(Wrappers.<SingleEnterpriseSettleSameAmountForSeveralMembersInCurrentMonth>lambdaQuery().eq(SingleEnterpriseSettleSameAmountForSeveralMembersInCurrentMonth::getUuid, r.getUuid()));
			} else if (r.getJobId() == 29) {
				String[] split = r.getMethodParamsValue().split(StrUtil.COMMA);
				jobName = String.format(j29, split[0]);
				count = singleMemberWithoutSuchTaskTypesInSeveralMonthsMapper.selectCount(Wrappers.<SingleMemberWithoutSuchTaskTypesInSeveralMonths>lambdaQuery().eq(SingleMemberWithoutSuchTaskTypesInSeveralMonths::getUuid, r.getUuid()));
			} else if (r.getJobId() == 30) {
				String[] split = r.getMethodParamsValue().split(StrUtil.COMMA);
				jobName = String.format(j30, split[0], split[1]);
				count = singleMemberSettleAmountOverrunInSeveralMonthsMapper.selectCount(Wrappers.<SingleMemberSettleAmountOverrunInSeveralMonths>lambdaQuery().eq(SingleMemberSettleAmountOverrunInSeveralMonths::getUuid, r.getUuid()));
			} else {
				jobName = r.getJobName();
			}
			r.setJobName(jobName);
			r.setEmpty(count == 0);

			LocalDateTime createTime = r.getCreateTime();
			String executeTime = r.getExecuteTime();
			int execTime = StrUtil.isBlank(executeTime) ? 0 : Integer.parseInt(executeTime);
			LocalDateTime plus = createTime.plus(execTime, ChronoUnit.MILLIS);
			r.setEndTime(plus);
		});

		return R.ok(page);
	}

}
