
package com.qunzhixinxi.hnqz.daemon.quartz.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.WmTaskType;
import com.qunzhixinxi.hnqz.daemon.quartz.service.WmTaskTypeService;
import lombok.AllArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;


/**
 * 任务类型打分表
 *
 * @author gaoyang
 * @date 2020-07-01 20:20:05
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmtasktype" )
public class WmTaskTypeController {

    private final WmTaskTypeService wmTaskTypeService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmTaskType 任务类型打分表
     * @return
     */
    @GetMapping("/page" )
    public R getWmTaskTypePage(Page page, WmTaskType wmTaskType) {
        return R.ok(wmTaskTypeService.page(page, Wrappers.query(wmTaskType)));
    }


    /**
     * 通过id查询任务类型打分表
     * @param id id
     * @return R
     */
    @GetMapping("/{id}" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmTaskTypeService.getById(id));
    }

    /**
     * 新增任务类型打分表
     * @param wmTaskType 任务类型打分表
     * @return R
     */
    @SysLog("新增任务类型打分表" )
    @PostMapping
    public R save(@RequestBody WmTaskType wmTaskType) {
		wmTaskType.setCreateTime(LocalDateTime.now());
		if(SecurityUtils.getRoles().contains(2))
		{
			wmTaskType.setStatus("0");
		}else{
			wmTaskType.setStatus("1");
		}
    	return R.ok(wmTaskTypeService.save(wmTaskType));
    }

    /**
     * 修改任务类型打分表
     * @param wmTaskType 任务类型打分表
     * @return R
     */
    @SysLog("修改任务类型打分表" )
    @PutMapping
    public R updateById(@RequestBody WmTaskType wmTaskType) {
		if(SecurityUtils.getRoles().contains(2))
		{
			WmTaskType wmTaskRule1=wmTaskType;
			wmTaskRule1.setId(null);
			wmTaskRule1.setStatus("1");
			wmTaskTypeService.save(wmTaskRule1);
		}else{
			wmTaskTypeService.updateById(wmTaskType);
		}
    	return R.ok();
    }

    /**
     * 通过id删除任务类型打分表
     * @param id id
     * @return R
     */
    @SysLog("通过id删除任务类型打分表" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('admin_wmtasktype_del')" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmTaskTypeService.removeById(id));
    }

	/**
	 * 获取药企列表
	 * @return 角色列表
	 */
	@GetMapping("/list")
	public R listWmScorePackage() {
		WmTaskType wmDaDrugEnt=new WmTaskType();
		wmDaDrugEnt.setDelFlag("0");
		wmDaDrugEnt.setEnableFlag("0");
		wmDaDrugEnt.setTaskTypeLevel("1");
		return R.ok(wmTaskTypeService.list(Wrappers.query(wmDaDrugEnt)));
	}
}
